#include <avr/eeprom.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>

#include "common.h"
#include "eeprom.h"
#include "extern.h"
#include "DS3234S.h"
#include "xitoa.h"

// init SPI
void init_SPI(void){
	SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPHA); // SPI Master Mode 0
	SPSR = _BV(SPI2X);
}

// init RTC
// init RTC must do after read eeprom
void init_DS3234S(void){
	//uint8_t eeprom_year;
	uint8_t temp_rtc;

	// RTC init
	rtc_write_bcd(RTC_CONTROL_ADDR, 0b00000000); // 1Hz, INT output
	rtc_write_bcd(RTC_CONTROL_STATUS_ADDR, (RTC_STATUS_OSF));

	// check rtc data
	rtc_read_all();

	//sec
	if((global_rtc_sec & 0b00001111) > 9){
		reset_rtc();
		return;
	}
	if((global_rtc_sec >> 4) > 5){
		reset_rtc();
		return;
	}
	temp_rtc = (global_rtc_sec >> 4) * 10 + (global_rtc_sec & 0b00001111);
	if(temp_rtc > 59){
		reset_rtc();
		return;
	}
	// min
	if((global_rtc_min & 0b00001111) > 9){
		reset_rtc();
		return;
	}
	if((global_rtc_min >> 4) > 5){
		reset_rtc();
		return;
	}
	temp_rtc = (global_rtc_min >> 4) * 10 + (global_rtc_min & 0b00001111);
	if(temp_rtc > 59){
		reset_rtc();
		return;
	}
	// hour
	if(global_rtc_hour & RTC_AMPM_FLAG){
		if((global_rtc_hour & 0b00001111) > 9){
			reset_rtc();
			return;
		}
		if(((global_rtc_hour >> 4) & 0b00000001) > 1){
			reset_rtc();
			return;
		}
		temp_rtc = ((global_rtc_hour >> 4) & 0b00000001) * 10 + (global_rtc_hour & 0b00001111);
		if(temp_rtc > 12){
			reset_rtc();
			return;
		}
	}else{
		if((global_rtc_hour & 0b00001111) > 9){
			reset_rtc();
			return;
		}
		if(((global_rtc_hour >> 4) & 0b00000011) > 2){
			reset_rtc();
			return;
		}
		temp_rtc = ((global_rtc_hour >> 4) & 0b00000011) * 10 + (global_rtc_hour & 0b00001111);
		if(temp_rtc > 24){
			reset_rtc();
			return;
		}
	}
	eep_update_ampm(global_config_data.ampm);
	change_hour_type(global_config_data.ampm);
	// day
	if(global_rtc_day > 7){
		reset_rtc();
		return;
	}
	// date
	if((global_rtc_date & 0b00001111) > 9){
		reset_rtc();
		return;
	}
	if((global_rtc_date >> 4) > 3){
		reset_rtc();
		return;
	}
	temp_rtc = (global_rtc_date >> 4) * 10 + (global_rtc_date & 0b00001111);
	if(temp_rtc > 31){
		reset_rtc();
		return;
	}
	// month
	if((global_rtc_month & 0b00001111) > 9){
		reset_rtc();
		return;
	}
	if(((global_rtc_month >> 4) & 0b00000001) > 1){
		reset_rtc();
		return;
	}
	temp_rtc = ((global_rtc_month >> 4) & 0b00000001) * 10 + (global_rtc_month & 0b00001111);
	if(temp_rtc > 12){
		reset_rtc();
		return;
	}
	// year
	if((global_rtc_year & 0b00001111) > 9){
		reset_rtc();
		return;
	}
	if((global_rtc_year >> 4) > 9){
		reset_rtc();
		return;
	}
	temp_rtc = (global_rtc_year >> 4) * 10 + (global_rtc_year & 0b00001111);
	if(temp_rtc > 99){
		reset_rtc();
		return;
	}
}

// check century flag in month address
void check_century(void){
	uint8_t century;
	uint8_t buf[3];

	// read manulally, prevent loop
	if((global_rtc_month & RTC_CENTURY_FLAG) != 0){
		// check year & month & date to prevent startup century flag issue
		if(global_rtc_year == 0 || global_rtc_month == 1 || global_rtc_date == 1){
			century = (global_config_data.century >> 4) * 10 + (global_config_data.century & 0b00001111);
			century++;
			if(century > 99){century = 0;}
			sprintf((char *)buf, "%02d", century);
			buf[0] -= 48;
			buf[1] -= 48;
			eep_update_century(century);
			// remove flag
			global_rtc_month &= (uint8_t)~RTC_CENTURY_FLAG;
			// write new month
			RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);
			SPDR = RTC_MONTH_ADDR  + RTC_WRITE;
			loop_until_bit_is_set(SPSR, SPIF);
			SPDR = global_rtc_month;
			loop_until_bit_is_set(SPSR, SPIF);
			RTC_CS_PORT |= _BV(RTC_CS);
		}else{
			// remove flag
			global_rtc_month &= (uint8_t)~RTC_CENTURY_FLAG;
			// write new month
			RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);
			SPDR = RTC_MONTH_ADDR  + RTC_WRITE;
			loop_until_bit_is_set(SPSR, SPIF);
			SPDR = global_rtc_month;
			loop_until_bit_is_set(SPSR, SPIF);
			RTC_CS_PORT |= _BV(RTC_CS);
		}
	}
}


// read rtc all data
void rtc_read_all(void){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	// start from sec
	SPDR = RTC_SEC_ADDR + RTC_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_sec = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_min = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_hour = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_day = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_date = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_month = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_year = SPDR;

	RTC_CS_PORT |= _BV(RTC_CS);

	check_century();
}

// read rtc time
void rtc_read_time(void){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	// start from sec
	SPDR = RTC_SEC_ADDR + RTC_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_sec = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_min = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_hour = SPDR;

	RTC_CS_PORT |= _BV(RTC_CS);
}

// read rtc date
void rtc_read_date(void){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	// start from day
	SPDR = RTC_DAY_ADDR + RTC_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_day = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_date = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_month = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_year = SPDR;

	RTC_CS_PORT |= _BV(RTC_CS);

	check_century();
}

// write rtc
void rtc_write_bcd(uint8_t addr, uint8_t data){
	uint8_t w_addr = 0;
	uint8_t str[3];
	uint8_t temp;

	temp = (data >> 4) * 10 + (data &0b00001111);

	switch(addr){
		case RTC_SEC_ADDR:
			if(temp < 0 || temp > 59){return;}
			break;

		case RTC_MIN_ADDR:
			if(temp < 0 || temp > 59){return;}
			break;

		// this hour must be 24hr format
		// & judge 24hr/12hr from EEPROM(global_config_data.ampm)
		case RTC_HOUR_ADDR:
			// check hour
			data &= (uint8_t)~(RTC_AMPM_FLAG);
			temp = ((data & 0b00110000) >> 4) * 10 + (data &0b00001111);
			if(temp < 0 || temp > 24){return;}

			//         00-01-02-03-04-05-06-07-08-09-10-11
			// 12hr AM 12-01-02-03-04-05-06-07-08-09-10-11
			if(global_config_data.ampm == HALF && temp >= 0 && temp < 12){
				if(temp == 0){data = 0b00010010;} // 12
				data |= RTC_AMPM_FLAG;
			//         12-13-14-15-16-17-18-19-20-21-22-23
			// 12hr PM 12-01-02-03-04-05-06-07-08-09-10-11
			}else if(global_config_data.ampm == HALF && temp >= 12 && temp < 24){
				if(temp != 12){temp -= 12;}
				sprintf((char *)str, "%02d", temp);
				str[0] -= 48;
				str[1] -= 48;
				data = (str[0] << 4) | str[1];
				data |= (RTC_AMPM_FLAG | RTC_PM_FLAG);
			}
			// 24hr do nothing
			break;

		case RTC_DAY_ADDR:
			if(temp < 1 || temp > 7){return;}
			break;

		case RTC_DATE_ADDR:
			if(temp < 0 || temp > 31){return;}
			break;

		case RTC_MONTH_ADDR:
			check_century();
			if(temp < 0 || temp > 12){return;}
			break;

		case RTC_YEAR_ADDR:
			if(temp < 0 || temp > 99){return;}
			break;

		case RTC_CONTROL_ADDR:
			break;

		case RTC_CONTROL_STATUS_ADDR:
			break;

		default:
			return;
	}

	w_addr = addr + RTC_WRITE;
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	RTC_CS_PORT |= _BV(RTC_CS);
}

void rtc_write_int(uint8_t addr, uint8_t data){
	uint8_t w_addr = 0;
	uint8_t str[3];
	uint8_t temp;

	sprintf((char *)str, "%02d", data);
	str[0] -= 48;
	str[1] -= 48;
	temp = data;
	data = (str[0] << 4) | str[1];

	switch(addr){
		case RTC_SEC_ADDR:
			if(temp < 0 || temp > 59){return;}
			break;

		case RTC_MIN_ADDR:
			if(temp < 0 || temp > 59){return;}
			break;

		// this hour must be 24hr format
		// & judge 24hr/12hr from EEPROM(global_config_data.ampm)
		case RTC_HOUR_ADDR:
			// check hour
			if(temp < 0 || temp > 24){return;}

			//         00-01-02-03-04-05-06-07-08-09-10-11
			// 12hr AM 12-01-02-03-04-05-06-07-08-09-10-11
			if(global_config_data.ampm == HALF && temp >= 0 && temp < 12){
				if(temp == 0){data = 0b00010010;} // 12
				data |= RTC_AMPM_FLAG;
			//         12-13-14-15-16-17-18-19-20-21-22-23
			// 12hr PM 12-01-02-03-04-05-06-07-08-09-10-11
			}else if(global_config_data.ampm == HALF && temp >= 12 && temp < 24){
				if(temp != 12){temp -= 12;}
				sprintf((char *)str, "%02d", temp);
				str[0] -= 48;
				str[1] -= 48;
				data = (str[0] << 4) | str[1];
				data |= (RTC_AMPM_FLAG | RTC_PM_FLAG);
			}
			// 24hr do nothing
			break;

		case RTC_DAY_ADDR:
			if(temp < 1 || temp > 7){return;}
			break;

		case RTC_DATE_ADDR:
			if(temp < 0 || temp > 31){return;}
			break;

		case RTC_MONTH_ADDR:
			check_century();
			if(temp < 0 || temp > 12){return;}
			break;

		case RTC_YEAR_ADDR:
			if(temp < 0 || temp > 99){return;}
			break;

		case RTC_CONTROL_ADDR:
			break;

		case RTC_CONTROL_STATUS_ADDR:
			break;

		default:
			return;
	}
	w_addr = addr + RTC_WRITE;
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	RTC_CS_PORT |= _BV(RTC_CS);
}

void rtc_write_time(uint8_t sec, uint8_t min, uint8_t hour){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	SPDR = RTC_SEC_ADDR + RTC_WRITE;
	loop_until_bit_is_set(SPSR, SPIF);

	// start from sec
	// sec
	SPDR = sec;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_sec = sec;

	// min
	SPDR = min;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_min = min;

	// hour
	SPDR = hour;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_hour = hour;

	RTC_CS_PORT |= _BV(RTC_CS);
}

void rtc_write_date(uint8_t day, uint8_t date, uint8_t month, uint8_t year){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	SPDR = RTC_DAY_ADDR + RTC_WRITE;
	loop_until_bit_is_set(SPSR, SPIF);

	// start from day
	// day
	SPDR = day;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_day = day;

	// date
	SPDR = date;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_date = date;

	// month
	SPDR = month;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_month = month;

	// year
	SPDR = year;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_year = year;

	RTC_CS_PORT |= _BV(RTC_CS);
}

void reset_rtc(void){
	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	SPDR = RTC_SEC_ADDR + RTC_WRITE;
	loop_until_bit_is_set(SPSR, SPIF);

	// sec
	SPDR = RTC_SEC_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_sec = RTC_SEC_DEFAULT;

	// min
	SPDR = RTC_MIN_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_min = RTC_MIN_DEFAULT;

	// hour
	SPDR = RTC_HOUR_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_hour = RTC_HOUR_DEFAULT;
	eep_update_ampm(FULL);

	// day
	SPDR = RTC_DAY_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_day = RTC_DAY_DEFAULT;

	// date
	SPDR = RTC_DATE_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_date = RTC_DATE_DEFAULT;

	// month
	SPDR = RTC_MONTH_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_month = RTC_MONTH_DEFAULT;

	// year
	SPDR = RTC_YEAR_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_year = RTC_YEAR_DEFAULT;

	RTC_CS_PORT |= _BV(RTC_CS);
}

void change_hour_type(uint8_t type){
	uint8_t hour;

	RTC_CS_PORT &= (uint8_t)~_BV(RTC_CS);

	// read hour
	SPDR = RTC_HOUR_ADDR + RTC_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_rtc_hour = SPDR;

	RTC_CS_PORT |= _BV(RTC_CS);

	// change
	// 12hr to 24hr
	if(type == FULL){
		if(!(global_rtc_hour & RTC_AMPM_FLAG)){return;}

		// 12hr @ PM to 24hr
		// 12-01-02-03-04-05-06-07-08-09-10-11
		// 12-13-14-15-16-17-18-19-20-21-22-23
		if(global_rtc_hour & RTC_AMPM_FLAG && global_rtc_hour & RTC_PM_FLAG){
			hour = ((global_rtc_hour & 0b00010000) >> 4) * 10 + (global_rtc_hour & 0b00001111);
			if(hour != 12){hour += 12;}
			eep_update_ampm(type);
			rtc_write_int(RTC_HOUR_ADDR, hour);

		// 12hr @ AM to 24hr
		// 12-01-02-03-04-05-06-07-08-09-10-11
		// 00-01-02-03-04-05-06-07-08-09-10-11
		}else if(global_rtc_hour & RTC_AMPM_FLAG && !(global_rtc_hour & RTC_PM_FLAG)){
			hour = ((global_rtc_hour & 0b00010000) >> 4) * 10 + (global_rtc_hour & 0b00001111);
			if(hour == 12){
				eep_update_ampm(type);
				rtc_write_int(RTC_HOUR_ADDR, 0);
			}else{
				eep_update_ampm(type);
				rtc_write_int(RTC_HOUR_ADDR, hour);
			}
		}
	// 24hr to 12hr
	}else if(type == HALF){
		if(global_rtc_hour & RTC_AMPM_FLAG){return;}

		hour = ((global_rtc_hour & 0b00110000) >> 4) * 10 + (global_rtc_hour & 0b00001111);
		eep_update_ampm(type);
		rtc_write_int(RTC_HOUR_ADDR, hour);
	}
}

void toggle_hour_type(void){
	if(global_config_data.ampm == FULL){
		change_hour_type(HALF);
	}else{
		change_hour_type(FULL);
	}
}
