#include <avr/io.h>
#include <stdio.h>

#include "common.h"
#include "eeprom.h"
#include "extern.h"
#include "LPS331.h"
#include "i2c.h"
#include "xitoa.h"

void init_LPS331(void){
	uint8_t status;

	// check status
	while(1){
		i2c_read(LPS331_ADDR, LPS331_WHO_AM_I, &status);
		if(status == LPS331_GOOD || status == LPS25H_GOOD){break;}
	}
	// write setting
	// max resolution
	if(status == LPS331_GOOD){
		i2c_write(LPS331_ADDR, LPS331_RES_CONF, 0x7A);
	}else if(status == LPS25H_GOOD){
		i2c_write(LPS331_ADDR, LPS331_RES_CONF, 0x0F);
	}
	// active mode and 1Hz output
	i2c_write(LPS331_ADDR, LPS331_CTRL_REG1, (LPS3310_CTRL_REG1_PD | LPS3310_CTRL_REG1_ODR0));

	// stabilize
	wait_ms(500);

	// initial access
	read_LPS331();
}

// read pressure & temperature
void read_LPS331(void){
	uint8_t data[5] = {0};
	int32_t pressure = 0;
	int32_t temperature = 0;

	// read raw data
	// to read multi data, add 0b10000000 to address
	i2c_read_multi(LPS331_ADDR, (LPS331_PRESS_POUT_XL_REH | LPS331_MULTI_READ), data, 5);

	// calc pressure
	if(data[2] & LPS331_PRESS_INVERT){
		data[0] = (uint8_t)~data[0];
		data[1] = (uint8_t)~data[1];
		data[2] = (uint8_t)~data[2];
		pressure = ((((uint32_t)data[2]) << 16) | (((uint32_t)data[1]) << 8) | ((uint32_t)data[0])) + 1;
		global_pressure = -((double)pressure) / 4096;
	}else{
		pressure = (((uint32_t)data[2]) << 16) | (((uint32_t)data[1]) << 8) | ((uint32_t)data[0]);
		global_pressure = ((double)pressure) / 4096;
	}

	// calc temperature
	if(data[4] & LPS331_TEMP_INVERT){
		data[3] = (uint8_t)~data[3];
		data[4] = (uint8_t)~data[4];
		temperature = (((uint32_t)data[4] << 8) | (uint32_t)data[3]) + 1;
		global_temperature = 42.5 - ((double)temperature) / 480.0;
	}else{
		temperature = (((uint32_t)data[4] << 8) | (uint32_t)data[3]);
		global_temperature = 42.5 + ((double)temperature) / 480.0;
	}
	// mmHg
	if(global_config_data.pres_type == MMHG){
		global_pressure = mmHg(global_pressure);
	}
	// Fahrenheit
	if(global_config_data.temp_type == FAH){
		global_temperature = fahrenheit(global_temperature);
	}
	// Kelvin
	if(global_config_data.temp_type == KEL){
		global_temperature += KELVIN;
	}
}

void toggle_temp_type(void){
	if(global_config_data.temp_type == CEL){
		eep_update_temp_type(FAH);
	}else if(global_config_data.temp_type == FAH){
		eep_update_temp_type(KEL);
	}else if(global_config_data.temp_type == KEL){
		eep_update_temp_type(CEL);
	}
}


void toggle_pres_type(void){
	if(global_config_data.pres_type == PA){
		eep_update_pres_type(MMHG);
	}else if(global_config_data.pres_type == MMHG){
		eep_update_pres_type(PA);
	}
}
