#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "ad.h"
#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "int.h"
#include "mode.h"
#include "uart.h"

// int init
void init_int(void){
	// INT4 for GPS 1pps & INT7 for RTC
	EIMSK |= _BV(INT4) | _BV(INT7);
	// RTC is clock up/down edge, GPS is clock up edge
	EICRB = _BV(ISC40) | _BV(ISC41) | _BV(ISC70);
}

// GPS 1pps
ISR(INT4_vect){
	uint8_t temp_hour;

	// update by 10sec @ sec == 5
	if(global_gpstime.valid == 1 && global_gpstime.time[5] == '4' &&  global_config_data.use_gps == 1){
		global_gpstime.valid = 0;

		// adjust timezone
		global_gpstime.time[0] -= 48;
		global_gpstime.time[1] -= 48;
		temp_hour = global_gpstime.time[0] * 10 + global_gpstime.time[1];
		temp_hour += global_config_data.gps_tz;

		// check 24hr ouverflow
		if(temp_hour >= 24){
			temp_hour -= 24;
		}

		// adjust 1pps timing
		// second must be 0 - 8, 9 is not good
		global_gpstime.time[5]++;

		// character to num
		global_gpstime.time[0] = temp_hour / 10;
		global_gpstime.time[1] = (temp_hour - global_gpstime.time[0] * 10);
		global_gpstime.time[2] -= 48;
		global_gpstime.time[3] -= 48;
		global_gpstime.time[4] -= 48;
		global_gpstime.time[5] -= 48;
		
		// write RTC
		rtc_write_bcd(RTC_SEC_ADDR,  (global_gpstime.time[4] << 4) | global_gpstime.time[5]);
		rtc_write_bcd(RTC_MIN_ADDR,  (global_gpstime.time[2] << 4) | global_gpstime.time[3]);
		rtc_write_bcd(RTC_HOUR_ADDR, (global_gpstime.time[0] << 4) | global_gpstime.time[1]);

		// update flag
		global_gps_count = 0;
		
		// signal LED
		toggle_gps_led();
	}
}

// RTC 1Hz int(clk up/down edge 0.5Hz)
ISR(INT7_vect){
	static uint32_t sec_count_sub = 0;

	// update rtc data
	rtc_read_all();

	// update mode
	update_mode(CURRENT_MODE);

	// check light sensor & do dimmer
	check_dimmer();

	// calc now sec
	gen_all_sec();

	// check turnoff
	check_turnoff();

	// check shuffle time
	check_shuffle_time();

	// chekc hvps
	check_hvps();

	// update operation time by 2hour
	sec_count_sub++;
	if(sec_count_sub > SEC_UPDATE_COUNT){
		sec_count_sub = 0;
		eep_update_sec_count(global_config_data.sec_count + SEC_UPDATE_INCREMENT);
	}

	// suppress LEDs
	off_alarm_led();
	off_gps_led();
	off_tx_led();
	off_rx_led();

	// reset WDT
	wdt_reset();
}
