#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include "ad.h"
#include "command.h"
#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "i2c.h"
#include "int.h"
#include "LPS331.h"
#include "port.h"
#include "timer.h"
#include "uart.h"
#include "xitoa.h"
#include "xor_shift.h"

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

// digit position 0->1->2->3->4->5->6->7
volatile uint8_t global_digit = 0;
// duty count
volatile uint8_t global_duty = 0;
// config eeprom map
volatile CONFIG_DATA global_config_eep_map __attribute__((section(".eeprom")));
// config SRAM data
volatile CONFIG_DATA global_config_data;
// digit duty
volatile uint8_t global_digit_duty[DIGIT_NUM] = {0};
// digit number
volatile uint8_t global_digit_num[DIGIT_NUM] = {0};
// digit PWM
volatile uint8_t global_digit_L_dot_pwm[DIGIT_NUM] = {0};
volatile uint8_t global_digit_R_dot_pwm[DIGIT_NUM] = {0};
// RTC raw data
volatile uint8_t global_rtc_hour = 0, global_rtc_min = 0, global_rtc_sec = 0;
volatile uint8_t global_rtc_year = 0, global_rtc_month = 0, global_rtc_date = 0, global_rtc_day = 0;
// Light sensor value
volatile uint16_t global_light_sensor = 0;
// last update count
volatile uint32_t global_gps_count = GPS_COUNT;
// record GPS time
volatile GPSTIME global_gpstime = {{0}};
// pressure & temperature data
volatile double global_pressure = 0;
volatile double global_temperature = 0;
// now all sec
volatile uint32_t global_now_all_sec = 0;
// amount of shuffle time (sec)
volatile uint32_t global_shuffle_all_sec[2] = {0};
// amount of turnoff time (sec) [0] is start [1] is end
volatile uint32_t global_turnoff_all_sec[2] = {0};
// xfade value
volatile uint16_t global_xfade_count = 0;
volatile uint16_t global_xfade_div_temp = 0;
volatile uint16_t global_xfade_edge     = 0;
volatile uint16_t global_xfade_term[XFADE_NUM] = {0};
// photo mode
volatile uint8_t global_photo_mode_pos = DIGIT_NUM + 1;
// config mode
volatile uint8_t global_config_mode = FALSE;
volatile uint8_t global_time_config_mode = FALSE;
volatile uint8_t global_date_config_mode = FALSE;
volatile uint8_t global_photo_config_mode = FALSE;
volatile uint8_t global_temp_digit[DIGIT_NUM] = {0};
volatile uint8_t global_temp_L_dot[DIGIT_NUM] = {0};
volatile uint8_t global_temp_R_dot[DIGIT_NUM] = {0};
// dot pwm direction
volatile uint8_t global_pwm_direction = UP;
// do shuffle flag
volatile uint8_t global_do_shuffle = DISABLE;
volatile uint8_t global_shuffle_continue = DISABLE;
volatile uint8_t global_shuffle_preset = DISABLE;
volatile uint8_t global_temp_shuffle_digit[DIGIT_NUM] = {0};
// hvps control
volatile uint8_t global_hvps_status = HVPS_ON;
volatile uint8_t global_temp_turnon = 0;
// test pattern
volatile uint8_t global_test_pattern = RANDOM;
volatile uint8_t global_test_pattern_num = 0;
volatile uint8_t global_test_pattern_dot = 0;

int main(void){
	wait_ms(500);
	cli();
	
	// init port
	init_port();
	// read setting
	read_eeprom_setting();
	// init a/d conv
	init_adc();
	// init SPI & RTC
	init_SPI();
	init_DS3234S();
	// init i2c & sensor
	init_i2c();
	init_LPS331();
	// init timer
	init_timer();
	// init int
	init_int();
	// xprintf output
	xfunc_out = uart1_put;
	// init uart
	init_uart();
	// set duty
	init_duty();
	// start HVPS
	on_hvps();

	sei();

	// start gps
	start_gps();

	on_power_led();

	// WDT start
	wdt_enable(WDTO_2S);

	// all reset
	if(bit_is_clear(ENC_PORT, ENC_SW_PIN)){
		reset_all_setting();
	}

	// loop
	while(1);
}
