/* Timer
** timer 0 nixie digit
** timer 1 PWM fot dot
** timer 2 PWM control
** timer 3 SW interrupts @10ms
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ad.h"
#include "common.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "LPS331.h"
#include "mode.h"
#include "timer.h"
#include "uart.h"
#include "xfade.h"
#include "xitoa.h"

void init_timer(void){
	// timer0 CTC interrupt @ 10 usec
	TIMSK |= _BV(OCIE0);
	TCCR0 = _BV(WGM01) | _BV(CS00); // 1/1
	OCR0 = 160;

	// timer1 pwm
	// enable pwm(16bit timer) for dot
	TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(WGM11); // 9 bit PWM
	TCCR1B = _BV(WGM12) | _BV(CS10);
	OCR1A = 0;
	OCR1B = 0;

	// timer2 control dot
	TIMSK |= _BV(OCIE2);
	TCCR2 = _BV(CS22); // 1/256

	// timer3 for SW interrupts(1ms:1000Hz)
	ETIMSK |= _BV(OCIE3A);
	TCCR3A = 0;
	TCCR3B = _BV(WGM32) | _BV(CS30); // 1/1
	OCR3A = 16000;
}

// timer0 control nixie
ISR(TIMER0_COMP_vect){
	// normal
	if(global_do_shuffle == DISABLE){
		if(global_duty < global_digit_duty[global_digit]){
			// when suppress do not active anode and all		
			if((global_digit_num[global_digit] == DIGIT_SUPPRESS) && (global_digit_L_dot_pwm[global_digit] == 0) && (global_digit_R_dot_pwm[global_digit] == 0)){
				// turn off anode
				NIXIE_ANODE_PORT = 0;
				// turn off cathode num
				NIXIE_CATHODE_NUM_PORT = DIGIT_SUPPRESS;
				// turn off cathode L dot
				NIXIE_CATHODE_L_DOT = 0;
				// turn off cathode R dot
				NIXIE_CATHODE_R_DOT = 0;
			// normal 
			}else{
				// turn on anode
				NIXIE_ANODE_PORT = _BV(global_digit);
				// turn on cathode num
				if(global_config_data.mode == MODE_TIME && global_config_data.use_xfade == ENABLE && global_config_mode == FALSE){
					do_xfade(global_digit_num[global_digit]);
				}else{
					NIXIE_CATHODE_NUM_PORT = global_digit_num[global_digit];
				}
				// turn on cathode L dot
				NIXIE_CATHODE_L_DOT = global_digit_L_dot_pwm[global_digit];
				// turn on cathode R dot
				NIXIE_CATHODE_R_DOT = global_digit_R_dot_pwm[global_digit];
			}
			global_duty++;
		}else if(global_duty < DUTY_CYCLE){
			// turn off anode
			NIXIE_ANODE_PORT = 0;
			// turn off cathode num
			NIXIE_CATHODE_NUM_PORT = DIGIT_SUPPRESS;
			// turn off cathode L dot
			NIXIE_CATHODE_L_DOT = 0;
			// turn off cathode R dot
			NIXIE_CATHODE_R_DOT = 0;
			global_duty++;
		}else{
			global_duty = 0;
			global_digit++;
			if(global_digit >= DIGIT_NUM){global_digit = 0;}
		}
	// shufle mode
	}else if(global_do_shuffle >= ENABLE){
		if(global_duty < DUTY_MAX){
			NIXIE_ANODE_PORT = _BV(global_digit);
			NIXIE_CATHODE_NUM_PORT = global_digit_num[global_digit];
			NIXIE_CATHODE_L_DOT = global_digit_L_dot_pwm[global_digit];
			NIXIE_CATHODE_R_DOT = global_digit_R_dot_pwm[global_digit];
			global_duty++;
		}else if(global_duty < DUTY_CYCLE){
			NIXIE_ANODE_PORT = 0;
			NIXIE_CATHODE_NUM_PORT = DIGIT_SUPPRESS;
			NIXIE_CATHODE_L_DOT = 0;
			NIXIE_CATHODE_R_DOT = 0;
			global_duty++;
		}else{
			global_duty = 0;
			global_digit++;
			if(global_digit >= DIGIT_NUM){global_digit = 0;}
		}
	}
}

// control nixie dot & shuffle mode
ISR(TIMER2_COMP_vect){
	static uint8_t temp_pwm = 0;
	static uint8_t i = 0;
	static uint8_t stop_count = 0;
	static uint8_t stop_flag[DIGIT_NUM] = {0};
	static uint8_t temp_position = 0;
	static uint8_t rand_count = 0;
	static uint16_t shuffle_count = 0;

	// in config do nothing
	if(global_config_mode == TRUE){return;}

	// dot control
	if(global_do_shuffle == FALSE){
		switch(global_config_data.mode){
		case MODE_TIME:

			switch(global_config_data.dot_mode){
			case DOT_PWM:
				if(global_pwm_direction == UP){ // direction is controlled @ mode(time & date)
					if(temp_pwm < DOT_PWM_MAX){
						temp_pwm++;
						//temp_pwm++;
						global_digit_R_dot_pwm[2] = temp_pwm;
						global_digit_R_dot_pwm[5] = temp_pwm;
					}else{
						global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
						global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
					}
				}else if(global_pwm_direction == DOWN){
					if(temp_pwm >= DOT_PWM_MIN){
						temp_pwm--;
						temp_pwm--;
						global_digit_R_dot_pwm[2] = temp_pwm;
						global_digit_R_dot_pwm[5] = temp_pwm;
					}else{
						global_digit_R_dot_pwm[2] = 0;
						global_digit_R_dot_pwm[5] = 0;
					}
				}else{
					global_digit_R_dot_pwm[2] = 0;
					global_digit_R_dot_pwm[5] = 0;
				}
				break;

			case DOT_BLINK:
				if(global_pwm_direction == UP){
					global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
					global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
				}else if(global_pwm_direction == DOWN){
					global_digit_R_dot_pwm[2] = 0;
					global_digit_R_dot_pwm[5] = 0;
				}else{
					global_digit_R_dot_pwm[2] = 0;
					global_digit_R_dot_pwm[5] = 0;
				}
				break;

			case DOT_ON:
				global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
				global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
				break;

			case DOT_OFF:
				global_digit_R_dot_pwm[2] = 0;
				global_digit_R_dot_pwm[5] = 0;
				break;

			default:
				global_digit_R_dot_pwm[2] = 0;
				global_digit_R_dot_pwm[5] = 0;
				break;
			}
			break;

		case MODE_DATE:
			
			switch(global_config_data.dot_mode){
			case DOT_PWM:
				if(global_pwm_direction == UP){
					if(temp_pwm < DOT_PWM_MAX){
						temp_pwm++;
						//temp_pwm++;
						global_digit_R_dot_pwm[3] = temp_pwm;
						global_digit_R_dot_pwm[5] = temp_pwm;
						global_digit_R_dot_pwm[7] = temp_pwm;
					}else{
						global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
						global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
						global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
					}
				}else if(global_pwm_direction == DOWN){
					if(temp_pwm >= DOT_PWM_MIN){
						temp_pwm--;
						//temp_pwm--;
						global_digit_R_dot_pwm[3] = temp_pwm;
						global_digit_R_dot_pwm[5] = temp_pwm;
						global_digit_R_dot_pwm[7] = temp_pwm;
					}else{
						global_digit_R_dot_pwm[3] = 0;
						global_digit_R_dot_pwm[5] = 0;
						global_digit_R_dot_pwm[7] = 0;
					}
				}else{
					global_digit_R_dot_pwm[3] = 0;
					global_digit_R_dot_pwm[5] = 0;
					global_digit_R_dot_pwm[7] = 0;
				}
				break;

			case DOT_BLINK:
				if(global_pwm_direction == UP){
				global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
					global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
					global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
				}else if(global_pwm_direction == DOWN){
					global_digit_R_dot_pwm[3] = 0;
					global_digit_R_dot_pwm[5] = 0;
					global_digit_R_dot_pwm[7] = 0;
				}else{
					global_digit_R_dot_pwm[3] = 0;
					global_digit_R_dot_pwm[5] = 0;
					global_digit_R_dot_pwm[7] = 0;
				}
				break;

			case DOT_ON:
				global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
				global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
				global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
				break;

			case DOT_OFF:
				global_digit_R_dot_pwm[3] = 0;
				global_digit_R_dot_pwm[5] = 0;
				global_digit_R_dot_pwm[7] = 0;
				break;

			default:
				global_digit_R_dot_pwm[3] = 0;
				global_digit_R_dot_pwm[5] = 0;
				global_digit_R_dot_pwm[7] = 0;
				break;
			}
			break;

		default:
			break;
		}
	}

	// shuffle control
	if(global_do_shuffle >= ENABLE){
		switch(global_do_shuffle){
		// shuffle digit & dot
		case SHUFFLE_MODE_1:
			global_digit_num[i] = rand_xor_shift()%10;
			global_digit_L_dot_pwm[i] = rand_xor_shift();
			global_digit_R_dot_pwm[i] = rand_xor_shift();
			break;

		// shuffle digit
		case SHUFFLE_MODE_2:
			if(i != 1){
				global_digit_num[i] = rand_xor_shift()%10;
				global_digit_L_dot_pwm[i] = 0;
				global_digit_R_dot_pwm[i] = 0;
			}else{
				global_digit_num[i] = DIGIT_SUPPRESS;
				global_digit_L_dot_pwm[i] = 0;
				global_digit_R_dot_pwm[i] = DOT_PWM_MAX;
			}
			break;

		// stop digit @ random
		case SHUFFLE_MODE_3:
			if(i != 1){
				if(stop_flag[i] == FALSE){
					global_digit_num[i] = rand_xor_shift()%10;
				}else if(global_shuffle_preset == TRUE){ // show preset number
					global_digit_num[i] = global_config_data.shuffle_digit[i];
					global_temp_shuffle_digit[i] = global_digit_num[i];
				}else{
					global_temp_shuffle_digit[i] = global_digit_num[i];
				}
				global_digit_L_dot_pwm[i] = 0;
				global_digit_R_dot_pwm[i] = 0;
			}else{
				global_digit_num[i] = DIGIT_SUPPRESS;
				global_digit_L_dot_pwm[i] = 0;
				global_digit_R_dot_pwm[i] = DOT_PWM_MAX;
			}
			stop_count++;
			if(stop_count > STOP_COUNT){
				stop_count = 0;
				if(rand_count < DIGIT_NUM){
					while(1){
						temp_position = rand_xor_shift()%8;
						if(stop_flag[temp_position] == FALSE){
							stop_flag[temp_position] = TRUE;
							break;
						}
					}
				}
				rand_count++;
			}
			break;

		// stop
		case SHUFFLE_MODE_4:
			stop_flag[i] = FALSE;
			temp_position = 0;
			stop_count = 0;
			rand_count = 0;
			global_shuffle_preset = DISABLE;
			break;

		// fade out
		case SHUFFLE_MODE_5:
			if(i <= temp_position){
				global_digit_num[i] = DIGIT_SUPPRESS;
				global_digit_L_dot_pwm[i] = 0;
				global_digit_R_dot_pwm[i] = 0;
			}
			stop_count++;
			if(stop_count > SCROLL_COUNT){
				stop_count = 0;
				temp_position++;
			}
			if(temp_position > DIGIT_NUM){
				stop_count = 0;
				temp_position = DIGIT_NUM;
				shuffle_count = 0;
				global_do_shuffle = SHUFFLE_MODE_6;
				break;
			}
			break;

		// fade in
		case SHUFFLE_MODE_6:
			if(i >= temp_position){
				if(i != 1){
					global_digit_num[i] = global_temp_shuffle_digit[i];
					global_digit_L_dot_pwm[i] = 0;
					global_digit_R_dot_pwm[i] = 0;
				}else{
					global_digit_num[i] = DIGIT_SUPPRESS;
					global_digit_L_dot_pwm[i] = 0;
					global_digit_R_dot_pwm[i] = DOT_PWM_MAX;
				}
			}
			stop_count++;
			if(stop_count > SCROLL_COUNT){
				stop_count = 0;
				temp_position--;
			}
			if(temp_position > DIGIT_NUM){
				stop_count = 0;
				temp_position = 0;
				shuffle_count = 0;
				global_do_shuffle = SHUFFLE_END;
				break;
			}
			break;

		default:
			break;
	}

		i++;
		if(i >= DIGIT_NUM){i=0;}

		shuffle_count++;
		if(shuffle_count > SHUFFLE_MODE_CHANGE){
			shuffle_count = 0;
			global_do_shuffle++;
		}
		if(global_do_shuffle >= SHUFFLE_END && global_shuffle_continue == FALSE){
			global_do_shuffle = DISABLE;
		}else if(global_do_shuffle >= SHUFFLE_END && global_shuffle_continue == TRUE){ // loop mode
			global_do_shuffle = ENABLE;
		}
	}	
}

void temp_digit_copy(void){
	global_temp_digit[0] = global_digit_num[0];
	global_temp_digit[1] = global_digit_num[1];
	global_temp_digit[2] = global_digit_num[2];
	global_temp_digit[3] = global_digit_num[3];
	global_temp_digit[4] = global_digit_num[4];
	global_temp_digit[5] = global_digit_num[5];
	global_temp_digit[6] = global_digit_num[6];
	global_temp_digit[7] = global_digit_num[7];
}

void temp_dot_copy(void){
	switch(global_config_data.mode){
	case MODE_TIME:
		if(!(global_rtc_hour & RTC_AMPM_FLAG)){ // 24hr
			global_temp_R_dot[0] = 0;
			global_temp_R_dot[1] = 0;
		}else if(global_rtc_hour & RTC_PM_FLAG){ // 12hr @ pm
			global_temp_R_dot[0] = 0;
			global_temp_R_dot[1] = DOT_PWM_MAX;
		}else{ // 12hr @ am
			global_temp_R_dot[0] = DOT_PWM_MAX;
			global_temp_R_dot[1] = 0;
		}
		global_temp_R_dot[2] = DOT_PWM_MAX;
		global_temp_R_dot[3] = 0;
		global_temp_R_dot[4] = 0;
		global_temp_R_dot[5] = DOT_PWM_MAX;
		global_temp_R_dot[6] = 0;
		global_temp_R_dot[7] = 0;

		global_temp_L_dot[0] = 0;
		global_temp_L_dot[1] = 0;
		global_temp_L_dot[2] = 0;
		global_temp_L_dot[3] = 0;
		global_temp_L_dot[4] = 0;
		global_temp_L_dot[5] = 0;
		global_temp_L_dot[6] = 0;
		global_temp_L_dot[7] = 0;
		break;

	case MODE_DATE:
		global_temp_R_dot[0] = 0;
		global_temp_R_dot[1] = 0;
		global_temp_R_dot[2] = 0;
		global_temp_R_dot[3] = DOT_PWM_MAX;
		global_temp_R_dot[4] = 0;
		global_temp_R_dot[5] = DOT_PWM_MAX;
		global_temp_R_dot[6] = 0;
		global_temp_R_dot[7] = DOT_PWM_MAX;

		global_temp_L_dot[0] = 0;
		global_temp_L_dot[1] = 0;
		global_temp_L_dot[2] = 0;
		global_temp_L_dot[3] = 0;
		global_temp_L_dot[4] = 0;
		global_temp_L_dot[5] = 0;
		global_temp_L_dot[6] = 0;
		global_temp_L_dot[7] = 0;
		break;

	case MODE_PHOTO:
		global_temp_R_dot[0] = global_config_data.photo_R_dot[0];
		global_temp_R_dot[1] = global_config_data.photo_R_dot[1];
		global_temp_R_dot[2] = global_config_data.photo_R_dot[2];
		global_temp_R_dot[3] = global_config_data.photo_R_dot[3];
		global_temp_R_dot[4] = global_config_data.photo_R_dot[4];
		global_temp_R_dot[5] = global_config_data.photo_R_dot[5];
		global_temp_R_dot[6] = global_config_data.photo_R_dot[6];
		global_temp_R_dot[7] = global_config_data.photo_R_dot[7];

		global_temp_L_dot[0] = global_config_data.photo_L_dot[0];
		global_temp_L_dot[1] = global_config_data.photo_L_dot[1];
		global_temp_L_dot[2] = global_config_data.photo_L_dot[2];
		global_temp_L_dot[3] = global_config_data.photo_L_dot[3];
		global_temp_L_dot[4] = global_config_data.photo_L_dot[4];
		global_temp_L_dot[5] = global_config_data.photo_L_dot[5];
		global_temp_L_dot[6] = global_config_data.photo_L_dot[6];
		global_temp_L_dot[7] = global_config_data.photo_L_dot[7];
		break;

	default:
		break;
	}
}

void change_temp_L_dot(uint8_t a){
	if(a & 0b10000000){
		global_temp_L_dot[0] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[0] = 0;
	}
	if(a & 0b01000000){
		global_temp_L_dot[1] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[1] = 0;
	}
	if(a & 0b00100000){
		global_temp_L_dot[2] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[2] = 0;
	}
	if(a & 0b00010000){
		global_temp_L_dot[3] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[3] = 0;
	}
	if(a & 0b00001000){
		global_temp_L_dot[4] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[4] = 0;
	}
	if(a & 0b00000100){
		global_temp_L_dot[5] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[5] = 0;
	}
	if(a & 0b00000010){
		global_temp_L_dot[6] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[6] = 0;
	}
	if(a & 0b00000001){
		global_temp_L_dot[7] = DOT_PWM_MAX;
	}else{
		global_temp_L_dot[7] = 0;
	}
}

void change_temp_R_dot(uint8_t a){
	if(a & 0b10000000){
		global_temp_R_dot[0] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[0] = 0;
	}
	if(a & 0b01000000){
		global_temp_R_dot[1] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[1] = 0;
	}
	if(a & 0b00100000){
		global_temp_R_dot[2] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[2] = 0;
	}
	if(a & 0b00010000){
		global_temp_R_dot[3] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[3] = 0;
	}
	if(a & 0b00001000){
		global_temp_R_dot[4] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[4] = 0;
	}
	if(a & 0b00000100){
		global_temp_R_dot[5] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[5] = 0;
	}
	if(a & 0b00000010){
		global_temp_R_dot[6] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[6] = 0;
	}
	if(a & 0b00000001){
		global_temp_R_dot[7] = DOT_PWM_MAX;
	}else{
		global_temp_R_dot[7] = 0;
	}
}

void update_temp_L_dot(uint8_t num, uint8_t power){
	if(num >= DIGIT_NUM){return;}
	if(power < DISABLE || power > ENABLE){return;}

	if(power == ENABLE){
		global_temp_L_dot[num] = DOT_PWM_MAX;
	}else if(power == DISABLE){
		global_temp_L_dot[num] = 0;
	}else{
		global_temp_L_dot[num] = 0;
	}
}

void update_temp_R_dot(uint8_t num, uint8_t power){
	if(num >= DIGIT_NUM){return;}
	if(power < DISABLE || power > ENABLE){return;}

	if(power == ENABLE){
		global_temp_R_dot[num] = DOT_PWM_MAX;
	}else if(power == DISABLE){
		global_temp_R_dot[num] = 0;
	}else{
		global_temp_R_dot[num] = 0;
	}
}

void toggle_dot(uint8_t i){
	if(i >= DIGIT_NUM){return;}

	if(global_temp_L_dot[i] != 0 && global_temp_R_dot[i] != 0){ // both on
		global_temp_L_dot[i] = 0;
		global_temp_R_dot[i] = 0;
	}else if(global_temp_L_dot[i] == 0 && global_temp_R_dot[i] == 0){ // both off
		global_temp_L_dot[i] = DOT_PWM_MAX;
		global_temp_R_dot[i] = 0;
	}else if(global_temp_L_dot[i] != 0 && global_temp_R_dot[i] == 0){ // L on R off
		global_temp_L_dot[i] = 0;
		global_temp_R_dot[i] = DOT_PWM_MAX;
	}else if(global_temp_L_dot[i] == 0 && global_temp_R_dot[i] != 0){ // L off R on
		global_temp_L_dot[i] = DOT_PWM_MAX;
		global_temp_R_dot[i] = DOT_PWM_MAX;
	}
}

// watch various, SW etc.
// SW input configure clock
ISR(TIMER3_COMPA_vect){
	// ENC SW
	static uint8_t enc_sw_old = _BV(ENC_SW_PIN); // ENC SW old
	static uint8_t enc_sw;                       // removed chattering ENC SW
	uint8_t enc_sw_new;                          // ENC SW new
	uint8_t temp;

	// for time temp
	static uint8_t temp_rtc_hour = 0;
	static uint8_t temp_hour = 0;
	static uint8_t temp_min  = 0;
	static uint8_t temp_sec  = 0;

	// for date temp
	static uint8_t temp_century = 0;
	static uint8_t temp_year    = 0;
	static uint8_t temp_month   = 0;
	static uint8_t temp_date    = 0;

	uint8_t temp_str[3];

	// encoder
	static uint8_t enc;

	// long push & skip flag
	static uint16_t long_push = 0;

	// check ENC SW
	enc_sw_new = ENC_PORT & _BV(ENC_SW_PIN);
	temp = ((uint8_t)~(enc_sw_old ^ enc_sw_new) & (enc_sw ^ enc_sw_new)); // twice same & afterward is different
	enc_sw_old = enc_sw_new;
	enc_sw ^= temp;

	// check ENC
	// http://elm-chan.org/docs/tec/te04.html
	temp = 0;
	if((ENC_PORT & _BV(ENC_A_PIN))){temp = 1;}  // phase A
	if((ENC_PORT & _BV(ENC_B_PIN))){temp += 2;} // phase B
	enc = (enc << 2) + temp;
	enc &= 15;

	// check long push
	if(bit_is_clear(ENC_PORT, ENC_SW_PIN)){
		long_push++;
	}else{
		if(long_push < LONG_PUSH_COUNT){
			long_push = 0;
		}
	}

	// long push operation
	if(long_push > LONG_PUSH_COUNT && bit_is_set(ENC_PORT, ENC_SW_PIN)){
		on_alarm_led();
		long_push = 0;

		switch(global_config_data.mode){
		case MODE_TIME:
			if(global_config_mode == FALSE){ // in normal
				toggle_hour_type();
			}else{ // inconfig
				if(global_time_config_mode == CONFIG_TIME_SEC){
					global_config_mode = FALSE;
					global_time_config_mode = CONFIG_TIME_NONE;
					normal_duty();
					rtc_write_int(RTC_SEC_ADDR, temp_sec);
					rtc_write_int(RTC_MIN_ADDR, temp_min);
					// 12hr PM
					if((temp_rtc_hour & RTC_AMPM_FLAG) && (temp_rtc_hour & RTC_PM_FLAG)){
						if(temp_hour == 12){
							rtc_write_int(RTC_HOUR_ADDR, temp_hour);
						}else{
							rtc_write_int(RTC_HOUR_ADDR, temp_hour + 12);
						}
					// 12hr AM
					}else if((temp_rtc_hour & RTC_AMPM_FLAG) && !(temp_rtc_hour & RTC_PM_FLAG)){
						if(temp_hour == 12){
							rtc_write_int(RTC_HOUR_ADDR, 0);
						}else{
							rtc_write_int(RTC_HOUR_ADDR, temp_hour);
						}
					// 24hr
					}else{
						rtc_write_int(RTC_HOUR_ADDR, temp_hour);
					}
				}
			}
			break;

		case MODE_DATE:
			if(global_config_mode == FALSE){ // in normal
				toggle_dot_mode();
			}else{ // inconfig
				if(global_date_config_mode == CONFIG_DATE_DATE){
					global_config_mode = FALSE;
					global_date_config_mode = CONFIG_DATE_NONE;
					normal_duty();
					rtc_write_int(RTC_DAY_ADDR, get_zeller_day(temp_century, temp_year, temp_month, temp_date));
					rtc_write_int(RTC_DATE_ADDR, temp_date);
					rtc_write_int(RTC_MONTH_ADDR, temp_month);
					rtc_write_int(RTC_YEAR_ADDR, temp_year);
					eep_update_century(temp_century);
				}
			}
			break;

		case MODE_PRESSURE:
			toggle_pres_type();
			break;

		case MODE_TEMPERATURE:
			toggle_temp_type();
			break;

		case MODE_SHUFFLE:
			do_shuffle_preset();
			break;

		case MODE_PHOTO:
			if(global_config_mode == FALSE){ // in normal
			}else{ // inconfig
				if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_16){
					global_config_mode = FALSE;
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_NONE;
					normal_duty();
					eep_update_photo_digit(global_temp_digit, sizeof(global_temp_digit),
										   global_temp_L_dot, sizeof(global_temp_L_dot),
										   global_temp_R_dot, sizeof(global_temp_R_dot));
				}
			}
			break;

		case MODE_TEST:
			change_test_pattern();
			break;

		default:
			eep_update_mode(MODE_TIME);
			break;
		}
		return;
	}
	
	// ENC SW operation
	if((enc_sw == 0) && (enc_sw_old == _BV(ENC_SW_PIN))){
		on_alarm_led();

		switch(global_config_data.mode){
		case MODE_TIME:
			if(global_config_mode == FALSE){ // in normal
				global_config_mode = TRUE;
				global_time_config_mode = CONFIG_TIME_HOUR;
				global_digit_duty[0] = DUTY_MAX;
				global_digit_duty[1] = DUTY_MAX;
				global_digit_duty[2] = DUTY_MIN;
				global_digit_duty[3] = DUTY_MIN;
				global_digit_duty[4] = DUTY_MIN;
				global_digit_duty[5] = DUTY_MIN;
				global_digit_duty[6] = DUTY_MIN;
				global_digit_duty[7] = DUTY_MIN;

				temp_rtc_hour = global_rtc_hour;

				temp_hour = global_digit_num[0] * 10 + global_digit_num[1];
				temp_min  = global_digit_num[3] * 10 + global_digit_num[4];
				temp_sec  = global_digit_num[6] * 10 + global_digit_num[7];

				temp_digit_copy();
				temp_dot_copy();
			}else{ // in config
				if(global_time_config_mode == CONFIG_TIME_HOUR){
					global_time_config_mode = CONFIG_TIME_MIN;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MAX;
					global_digit_duty[4] = DUTY_MAX;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_time_config_mode == CONFIG_TIME_MIN){
					global_time_config_mode = CONFIG_TIME_SEC;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MAX;
					global_digit_duty[7] = DUTY_MAX;
				}else if(global_time_config_mode == CONFIG_TIME_SEC){
					global_config_mode = FALSE;
					global_time_config_mode = CONFIG_TIME_NONE;
					normal_duty();
				}else{
					global_config_mode = FALSE;
					global_time_config_mode = CONFIG_TIME_NONE;
					normal_duty();
				}
			}
			break;

		case MODE_DATE:
			if(global_config_mode == FALSE){ // in normal
				global_config_mode = TRUE;
				global_date_config_mode = CONFIG_DATE_CENTURY;
				global_digit_duty[0] = DUTY_MAX;
				global_digit_duty[1] = DUTY_MAX;
				global_digit_duty[2] = DUTY_MIN;
				global_digit_duty[3] = DUTY_MIN;
				global_digit_duty[4] = DUTY_MIN;
				global_digit_duty[5] = DUTY_MIN;
				global_digit_duty[6] = DUTY_MIN;
				global_digit_duty[7] = DUTY_MIN;

				temp_century = global_digit_num[0] * 10 + global_digit_num[1];
				temp_year    = global_digit_num[2] * 10 + global_digit_num[3];
				temp_month   = global_digit_num[4] * 10 + global_digit_num[5];
				temp_date    = global_digit_num[6] * 10 + global_digit_num[7];

				temp_digit_copy();
				temp_dot_copy();
			}else{ // in config
				if(global_date_config_mode == CONFIG_DATE_CENTURY){
					global_date_config_mode = CONFIG_DATE_YEAR;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MAX;
					global_digit_duty[3] = DUTY_MAX;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_date_config_mode == CONFIG_DATE_YEAR){
					global_date_config_mode = CONFIG_DATE_MONTH;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MAX;
					global_digit_duty[5] = DUTY_MAX;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_date_config_mode == CONFIG_DATE_MONTH){
					global_date_config_mode = CONFIG_DATE_DATE;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MAX;
					global_digit_duty[7] = DUTY_MAX;
				}else if(global_date_config_mode == CONFIG_DATE_DATE){
					global_config_mode = FALSE;
					global_date_config_mode = CONFIG_DATE_NONE;
					normal_duty();
				}else{
					global_config_mode = FALSE;
					global_date_config_mode = CONFIG_DATE_NONE;
					normal_duty();
				}
				
			}
			break;

		case MODE_PRESSURE:
			toggle_pres_type();
			break;

		case MODE_TEMPERATURE:
			toggle_temp_type();
			break;

		case MODE_SHUFFLE:
			do_shuffle();
			break;

		case MODE_PHOTO:
			if(global_config_mode == FALSE){ // in normal
				global_config_mode = TRUE;
				global_photo_config_mode = CONFIG_PHOTO_DIGIT_1;
				global_digit_duty[0] = DUTY_MAX;
				global_digit_duty[1] = DUTY_MIN;
				global_digit_duty[2] = DUTY_MIN;
				global_digit_duty[3] = DUTY_MIN;
				global_digit_duty[4] = DUTY_MIN;
				global_digit_duty[5] = DUTY_MIN;
				global_digit_duty[6] = DUTY_MIN;
				global_digit_duty[7] = DUTY_MIN;

				temp_digit_copy();
				temp_dot_copy();
			}else{ // in config
				if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_1){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_2;
					global_digit_duty[0] = DUTY_MAX;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(0, 1);
					update_temp_R_dot(0, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_2){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_3;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MAX;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_3){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_4;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MAX;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(1, 1);
					update_temp_R_dot(1, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_4){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_5;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MAX;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_5){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_6;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MAX;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(2, 1);
					update_temp_R_dot(2, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_6){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_7;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MAX;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_7){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_8;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MAX;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(3, 1);
					update_temp_R_dot(3, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_8){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_9;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MAX;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_9){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_10;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MAX;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(4, 1);
					update_temp_R_dot(4, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_10){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_11;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MAX;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_11){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_12;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MAX;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(5, 1);
					update_temp_R_dot(5, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_12){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_13;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MAX;
					global_digit_duty[7] = DUTY_MIN;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_13){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_14;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MAX;
					global_digit_duty[7] = DUTY_MIN;
					update_temp_L_dot(6, 1);
					update_temp_R_dot(6, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_14){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_15;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MAX;
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_15){
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_16;
					global_digit_duty[0] = DUTY_MIN;
					global_digit_duty[1] = DUTY_MIN;
					global_digit_duty[2] = DUTY_MIN;
					global_digit_duty[3] = DUTY_MIN;
					global_digit_duty[4] = DUTY_MIN;
					global_digit_duty[5] = DUTY_MIN;
					global_digit_duty[6] = DUTY_MIN;
					global_digit_duty[7] = DUTY_MAX;
					update_temp_L_dot(7, 1);
					update_temp_R_dot(7, 1);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_16){
					global_config_mode = FALSE;
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_NONE;
					normal_duty();
				}else{
					global_config_mode = FALSE;
					global_photo_config_mode = CONFIG_PHOTO_DIGIT_NONE;
					normal_duty();
				}
			}
			break;

		case MODE_TEST:
			change_test_pattern();
			break;

		default:
			eep_update_mode(MODE_TIME);
			break;
		}
	}

	// ENC rotate operation
	// ENC1
	switch(enc){
	case 0x7: // decrement
		on_alarm_led();

		switch(global_config_data.mode){
		case MODE_TIME:
			if(global_config_mode == FALSE){ // in normal
				decrease_mode();
			}else{ // in config
				if(global_time_config_mode == CONFIG_TIME_HOUR){ // hour
					// 24hr
					if(global_config_data.ampm == FULL){
						if(temp_hour == 0){
							temp_hour = 23;
						}else{
							temp_hour--;
						}
						sprintf((char *)temp_str, "%02d", temp_hour);
						global_temp_digit[0] = temp_str[0] - 48;
						global_temp_digit[1] = temp_str[1] - 48;
						temp_rtc_hour = global_temp_digit[0] << 4 | global_temp_digit[1];
						update_mode(CURRENT_MODE);
					// 12hr
					}else{
						// 12hr PM 12<-01<-02<-03<-04<-05<-06<-07<-08<-09<-10<-11
						if(temp_rtc_hour & RTC_PM_FLAG){
							if(temp_hour == 12){
								temp_hour = 11;
								temp_rtc_hour &= (uint8_t)~RTC_PM_FLAG;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else if(temp_hour == 1){
								temp_hour = 12;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else{
								temp_hour--;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}
						// 12hr AM 12<-01<-02<-03<-04<-05<-06<-07<-08<-09<-10<-11
						}else{
							if(temp_hour == 12){
								temp_hour = 11;
								temp_rtc_hour |= RTC_PM_FLAG;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else if(temp_hour == 1){
								temp_hour = 12;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else{
								temp_hour--;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}
						}
					}
				}else if(global_time_config_mode == CONFIG_TIME_MIN){ // min
					if(temp_min == 0){
						temp_min = 59;
					}else{
						temp_min--;
					}
					sprintf((char *)temp_str, "%02d", temp_min);
					global_temp_digit[3] = temp_str[0] - 48;
					global_temp_digit[4] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_time_config_mode == CONFIG_TIME_SEC){ // sec
					if(temp_sec == 0){
						temp_sec = 59;
					}else{
						temp_sec--;
					}
					sprintf((char *)temp_str, "%02d", temp_sec);
					global_temp_digit[6] = temp_str[0] - 48;
					global_temp_digit[7] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_DATE:
			if(global_config_mode == FALSE){ // in normal
				decrease_mode();
			}else{ // in config
				if(global_date_config_mode == CONFIG_DATE_CENTURY){ // century
					if(temp_century == 0){
						temp_century = 99;
					}else{
						temp_century--;
					}
					sprintf((char *)temp_str, "%02d", temp_century);
					global_temp_digit[0] = temp_str[0] - 48;
					global_temp_digit[1] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_YEAR){ // year
					if(temp_year == 0){
						temp_year = 99;
					}else{
						temp_year--;
					}
					sprintf((char *)temp_str, "%02d", temp_year);
					global_temp_digit[2] = temp_str[0] - 48;
					global_temp_digit[3] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_MONTH){ // month
					if(temp_month == 1){
						temp_month = 12;
					}else{
						temp_month--;
					}
					sprintf((char *)temp_str, "%02d", temp_month);
					global_temp_digit[4] = temp_str[0] - 48;
					global_temp_digit[5] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_DATE){ // date
					if(temp_date == 1){
						temp_date = get_monthend(temp_century, temp_year, temp_month);
					}else{
						temp_date--;
					}
					sprintf((char *)temp_str, "%02d", temp_date);
					global_temp_digit[6] = temp_str[0] - 48;
					global_temp_digit[7] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_PRESSURE:
			if(global_config_mode == FALSE){ // in normal
				decrease_mode();
			}else{ // in config
			}
			break;

		case MODE_TEMPERATURE:
			if(global_config_mode == FALSE){ // in normal
				decrease_mode();
			}else{ // in config
			}
			break;

		case MODE_SHUFFLE:
			if(global_config_mode == FALSE){ // in normal
				decrease_mode();
			}else{ // in config
			}
			break;

		case MODE_PHOTO:
			if(global_config_mode == FALSE){ // in normal
			decrease_mode();
			}else{ // in config
				if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_1){
					if(global_temp_digit[0] == 0){
						global_temp_digit[0] = 10;
					}else{
						global_temp_digit[0]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_2){
					toggle_dot(0);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_3){
					if(global_temp_digit[1] == 0){
						global_temp_digit[1] = 10;
					}else{
						global_temp_digit[1]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_4){
					toggle_dot(1);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_5){
					if(global_temp_digit[2] == 0){
						global_temp_digit[2] = 10;
					}else{
						global_temp_digit[2]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_6){
					toggle_dot(2);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_7){
					if(global_temp_digit[3] == 0){
						global_temp_digit[3] = 10;
					}else{
						global_temp_digit[3]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_8){
					toggle_dot(3);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_9){
					if(global_temp_digit[4] == 0){
						global_temp_digit[4] = 10;
					}else{
						global_temp_digit[4]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_10){
					toggle_dot(4);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_11){
					if(global_temp_digit[5] == 0){
						global_temp_digit[5] = 10;
					}else{
						global_temp_digit[5]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_12){
					toggle_dot(5);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_13){
					if(global_temp_digit[6] == 0){
						global_temp_digit[6] = 10;
					}else{
						global_temp_digit[6]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_14){
					toggle_dot(6);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_15){
					if(global_temp_digit[7] == 0){
						global_temp_digit[7] = 10;
					}else{
						global_temp_digit[7]--;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_16){
					toggle_dot(7);
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_TEST:
			if(global_config_mode == FALSE){ // in normal
				if(global_test_pattern == MANUAL){
					global_test_pattern_dot++;
					if(global_test_pattern_dot > NONE){global_test_pattern_dot = BOTH;}
					update_mode(CURRENT_MODE);
				}else{
					decrease_mode();
				}
			}else{ // in config
			}
			break;

		default:
			eep_update_mode(MODE_TIME);
			break;
		}
		break;

	case 0xd: // increment
		on_alarm_led();

		switch(global_config_data.mode){
		case MODE_TIME:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
				// hour
				if(global_time_config_mode == CONFIG_TIME_HOUR){
					// 24hr
					if(global_config_data.ampm == FULL){
						if(temp_hour == 23){
							temp_hour = 0;
						}else{
							temp_hour++;
						}
						sprintf((char *)temp_str, "%02d", temp_hour);
						global_temp_digit[0] = temp_str[0] - 48;
						global_temp_digit[1] = temp_str[1] - 48;
						temp_rtc_hour = global_temp_digit[0] << 4 | global_temp_digit[1];
						update_mode(CURRENT_MODE);
					// 12hr
					}else{
						// 12hr PM 12->01->02->03->04->05->06->07->08->09->10->11
						if(temp_rtc_hour & RTC_PM_FLAG){
							if(temp_hour == 12){
								temp_hour = 1;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else if(temp_hour == 11){
								temp_hour = 12;
								temp_rtc_hour &= (uint8_t)~RTC_PM_FLAG;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else{
								temp_hour++;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}
						// 12hr AM 12->01->02->03->04->05->06->07->08->09->10->11
						}else{
							if(temp_hour == 12){
								temp_hour = 1;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else if(temp_hour == 11){
								temp_hour = 12;
								temp_rtc_hour |= RTC_PM_FLAG;
								change_temp_R_dot(0b01000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | RTC_PM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}else{
								temp_hour++;
								change_temp_R_dot(0b10000000);
								sprintf((char *)temp_str, "%02d", temp_hour);
								global_temp_digit[0] = temp_str[0] - 48;
								global_temp_digit[1] = temp_str[1] - 48;
								temp_rtc_hour = RTC_AMPM_FLAG | global_temp_digit[0] << 4 | global_temp_digit[1];
								update_mode(CURRENT_MODE);
							}
						}
					}
				}else if(global_time_config_mode == CONFIG_TIME_MIN){ // min
					if(temp_min == 59){
						temp_min = 0;
					}else{
						temp_min++;
					}
					sprintf((char *)temp_str, "%02d", temp_min);
					global_temp_digit[3] = temp_str[0] - 48;
					global_temp_digit[4] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_time_config_mode == CONFIG_TIME_SEC){ // sec
					if(temp_sec == 59){
						temp_sec = 0;
					}else{
						temp_sec++;
					}
					sprintf((char *)temp_str, "%02d", temp_sec);
					global_temp_digit[6] = temp_str[0] - 48;
					global_temp_digit[7] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_DATE:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
				if(global_date_config_mode == CONFIG_DATE_CENTURY){ // century
					if(temp_century == 99){
						temp_century = 0;
					}else{
						temp_century++;
					}
					sprintf((char *)temp_str, "%02d", temp_century);
					global_temp_digit[0] = temp_str[0] - 48;
					global_temp_digit[1] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_YEAR){ // year
					if(temp_year == 99){
						temp_year = 0;
					}else{
						temp_year++;
					}
					sprintf((char *)temp_str, "%02d", temp_year);
					global_temp_digit[2] = temp_str[0] - 48;
					global_temp_digit[3] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_MONTH){ // month
					if(temp_month == 12){
						temp_month = 1;
					}else{
						temp_month++;
					}
					sprintf((char *)temp_str, "%02d", temp_month);
					global_temp_digit[4] = temp_str[0] - 48;
					global_temp_digit[5] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}else if(global_date_config_mode == CONFIG_DATE_DATE){ // date
					if(temp_date == get_monthend(temp_century, temp_year, temp_month)){
						temp_date = 1;
					}else{
						temp_date++;
					}
					sprintf((char *)temp_str, "%02d", temp_date);
					global_temp_digit[6] = temp_str[0] - 48;
					global_temp_digit[7] = temp_str[1] - 48;
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_PRESSURE:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
			}
			break;

		case MODE_TEMPERATURE:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
			}
			break;

		case MODE_SHUFFLE:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
			}
			break;

		case MODE_PHOTO:
			if(global_config_mode == FALSE){ // in normal
				increase_mode();
			}else{
				if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_1){
					if(global_temp_digit[0] >= 10){
						global_temp_digit[0] = 0;
					}else{
						global_temp_digit[0]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_2){
					toggle_dot(0);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_3){
					if(global_temp_digit[1] >= 10){
						global_temp_digit[1] = 0;
					}else{
						global_temp_digit[1]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_4){
					toggle_dot(1);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_5){
					if(global_temp_digit[2] >= 10){
						global_temp_digit[2] = 0;
					}else{
						global_temp_digit[2]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_6){
					toggle_dot(2);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_7){
					if(global_temp_digit[3] >= 10){
						global_temp_digit[3] = 0;
					}else{
						global_temp_digit[3]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_8){
					toggle_dot(3);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_9){
					if(global_temp_digit[4] >= 10){
						global_temp_digit[4] = 0;
					}else{
						global_temp_digit[4]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_10){
					toggle_dot(4);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_11){
					if(global_temp_digit[5] >= 10){
						global_temp_digit[5] = 0;
					}else{
						global_temp_digit[5]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_12){
					toggle_dot(5);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_13){
					if(global_temp_digit[6] >= 10){
						global_temp_digit[6] = 0;
					}else{
						global_temp_digit[6]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_14){
					toggle_dot(6);
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_15){
					if(global_temp_digit[7] >= 10){
						global_temp_digit[7] = 0;
					}else{
						global_temp_digit[7]++;
					}
					update_mode(CURRENT_MODE);
				}else if(global_photo_config_mode == CONFIG_PHOTO_DIGIT_16){
					toggle_dot(7);
					update_mode(CURRENT_MODE);
				}
			}
			break;

		case MODE_TEST:
			if(global_config_mode == FALSE){ // in normal
				if(global_test_pattern == MANUAL){
					global_test_pattern_num++;
					if(global_test_pattern_num > 9){global_test_pattern_num = 0;}
					update_mode(CURRENT_MODE);
				}else{
					increase_mode();
				}
			}else{
			}
			break;

		default:
			eep_update_mode(MODE_TIME);
			break;
		}
		break;

	default:
		break;
	}
}
