#include <avr/io.h>
#include <avr/interrupt.h>

#include "ad.h"
#include "common.h"
#include "config.h"
#include "extern.h"
#include "xitoa.h"

// init AD converter
void init_adc(void){
	ADMUX = 0;
	//ADCSRA = _BV(ADEN) | _BV(ADPS0) | _BV(ADPS1) | _BV(ADPS2); // 1/128
	//ADCSRA = _BV(ADEN) | _BV(ADPS1) | _BV(ADPS2);              // 1/64
	ADCSRA = _BV(ADEN) | _BV(ADPS0) | _BV(ADPS2);              // 1/32
	//ADCSRA = _BV(ADEN) | _BV(ADPS2);                           // 1/16
	//ADCSRA = _BV(ADEN) | _BV(ADPS0) | _BV(ADPS1);              // 1/8
	//ADCSRA = _BV(ADEN) | _BV(ADPS1);                           // 1/4
	//ADCSRA = _BV(ADEN) | _BV(ADPS0);                           // 1/2
	//ADCSRA = _BV(ADEN);                                        // 1/1

	// stabilize
	get_light_sensor_val();
}

// single end input AD return 0-1023
uint16_t exec_adc(uint8_t pin){
	uint16_t adL = 0;
	uint16_t adH = 0;

	if(pin > 7){return 0;}               // only single end input
	ADMUX = pin;                         // select pin
	ADCSRA &= (uint8_t)~_BV(ADIF);       // disable flag
	ADCSRA |= _BV(ADSC);                 // a/d start
	loop_until_bit_is_set(ADCSRA, ADIF); // while done it
	adL = ADCL;                          // LSB 8bit
	adH = ADCH;                          // MSB 8bit(2bit)
	return (adH << 8) + adL;
}

// get A/D from S9648
void get_light_sensor_val(void){
	global_light_sensor = exec_adc(LIGHT_SENSOR_PIN);
}

// do dimmer
void check_dimmer(void){
	// not in config
	if(global_config_mode == CONFIG_NONE && global_config_data.use_light_sensor == ENABLE){
		get_light_sensor_val();
		// do dimmer
		if(global_config_data.light_sensor_th > global_light_sensor){
			// turnoff
			if(global_config_data.light_sensor_hvps == TRUE){
				global_hvps_status &= (uint8_t)~HVPS_SENSOR;
				minimum_duty();
			// dimmer
			}else{
				global_hvps_status |= HVPS_SENSOR;
				minimum_duty();
			}
		// return normal duty
		}else{
			global_hvps_status |= HVPS_SENSOR;
			normal_duty();
		}
	// in config, do not dimmer
	}else{
	}
}
