#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "extern.h"
#include "uart.h"
#include "xitoa.h"

// wait utility
void wait_ms(uint16_t t){
	while(t--){_delay_ms(1);}
}

void wait_us(uint16_t t){
	while(t--){_delay_us(1);}
	}

// alarm LED(red)
void on_alarm_led(void){
	ALARM_LED_PORT |= _BV(ALARM_LED);
}

void off_alarm_led(void){
	ALARM_LED_PORT &= (uint8_t)~_BV(ALARM_LED);
}

void toggle_alarm_led(void){
	ALARM_LED_PORT ^= _BV(ALARM_LED);
}

// power LED(green)
void on_power_led(void){
	POWER_LED_PORT |= _BV(POWER_LED);
}

void off_power_led(void){
	POWER_LED_PORT &= (uint8_t)~_BV(POWER_LED);
}

void toggle_power_led(void){
	POWER_LED_PORT ^= _BV(POWER_LED);
}

// transmit LED(blue)
void on_tx_led(void){
	TX_LED_PORT |= _BV(TX_LED);
}

void off_tx_led(void){
	TX_LED_PORT &= (uint8_t)~_BV(TX_LED);
}

void toggle_tx_led(void){
	TX_LED_PORT ^= _BV(TX_LED);
}

// reccive LED(yellow)
void on_rx_led(void){
	RX_LED_PORT |= _BV(RX_LED);
}

void off_rx_led(void){
	RX_LED_PORT &= (uint8_t)~_BV(RX_LED);
}

void toggle_rx_led(void){
	RX_LED_PORT ^= _BV(RX_LED);
}

// GPS LED(blue)
void on_gps_led(void){
	GPS_LED_PORT |= _BV(GPS_LED);
}

void off_gps_led(void){
	GPS_LED_PORT &= (uint8_t)~_BV(GPS_LED);
}

void toggle_gps_led(void){
	GPS_LED_PORT ^= _BV(GPS_LED);
}

// GPS control
void start_gps(void){
	GPS_PS_PORT |= _BV(GPS_PS);
	global_gps_count = GPS_COUNT;

	xfunc_out = uart0_put;
	xprintf(PSTR("$PMTK101*32\n"));      // hot start
	//xprintf(PSTR("$PMTK251,9600*17\n")); // 9600 baud cannot change...
	//xprintf(PSTR("$PMTK286,1*23\n"));    // enable AIC
	//xprintf(PSTR("$PMTK869,1,1*35\n"));    // enable easy
	//xprintf(PSTR("$PMTK220,200*2C\n"));  // 5Hz cannot change...
	xfunc_out = uart1_put;
}

void stop_gps(void){
	GPS_PS_PORT &= (uint8_t)~_BV(GPS_PS);
	global_gps_count = GPS_COUNT;
}

void reset_gps(void){
	GPS_PS_PORT |= _BV(GPS_PS);
	global_gps_count = GPS_COUNT;

	xfunc_out = uart0_put;
	//xprintf(PSTR("$PMTK103*30\n"));      // cold start
	xprintf(PSTR("$PMTK104*37\n"));      // full cold start(factory reset)
	//xprintf(PSTR("$PMTK251,9600*17\n")); // 9600 baud
	//xprintf(PSTR("$PMTK286,1*23\n"));    // enable AIC
	//xprintf(PSTR("$PMTK869,1,1*35\n"));    // enable easy
	xfunc_out = uart1_put;
}

// hvps control
void on_hvps(void){
	HVPS_SW_PORT &= (uint8_t)~_BV(HVPS_SW);
}

void off_hvps(void){
	HVPS_SW_PORT |= _BV(HVPS_SW);
}

// init duty
void init_duty(void){
	uint8_t i;

	for(i=0; i<DIGIT_NUM; i++){
		global_digit_duty[i] = DUTY_MAX;
		global_digit_L_dot_pwm[i] = 0;
		global_digit_R_dot_pwm[i] = 0;
	}
}

// normalize duty
void normal_duty(void){
	uint8_t i;

	for(i=0; i<DIGIT_NUM; i++){
		global_digit_duty[i] = global_config_data.duty;
	}

	// return cross fade timing
	global_xfade_edge = XFADE_CYCLE - global_config_data.xfade_div * XFADE_NUM - XFADE_DIM * (DUTY_MAX - global_config_data.duty);
	for(i=0; i<XFADE_NUM; i++){
		global_xfade_term[i] = global_xfade_edge + global_config_data.xfade_div * (i + 1);
	}
}

// minimize duty
void minimum_duty(void){
	uint8_t i;

	// set minimum temporarily
	for(i=0; i<DIGIT_NUM; i++){
		global_digit_duty[i] = DUTY_MIN;
	}

	// set cross fade timing temporarily
	//global_xfade_edge = XFADE_CYCLE - global_config_data.xfade_div * XFADE_NUM - XFADE_DIM * (DUTY_MAX - DUTY_MIN);
	//global_xfade_edge = XFADE_CYCLE - XFADE_DIV_MIN * XFADE_NUM - XFADE_DIM * (DUTY_MAX - DUTY_MIN);
	global_xfade_edge = XFADE_CYCLE - XFADE_DIV_SEN * XFADE_NUM - XFADE_DIM * (DUTY_MAX - DUTY_MIN);
	for(i=0; i<XFADE_NUM; i++){
		global_xfade_term[i] = global_xfade_edge + global_config_data.xfade_div * (i + 1);
	}
}

void show_op_time(void){
	xprintf(PSTR("OPERATION TIME=%d [sec] \n"), global_config_data.sec_count);
}

// reset all setting & show zero
void reset_all_setting(void){
	uint8_t i;

	// stop WDT
	wdt_disable();

	reset_gps();

	// stop interrupt
	cli();

	reset_eeprom_setting();
	reset_rtc();

	// on LEDs
	on_power_led();
	on_alarm_led();
	on_tx_led();
	on_rx_led();
	on_gps_led();

	// show zero loop
	while(1){
		for(i=0; i<DIGIT_NUM; i++){
			NIXIE_ANODE_PORT = _BV(i);
			NIXIE_CATHODE_NUM_PORT = 0;
			NIXIE_CATHODE_L_DOT = DOT_PWM_MAX;
			NIXIE_CATHODE_R_DOT = DOT_PWM_MAX;
			wait_us(50);

			NIXIE_ANODE_PORT = 0;
			NIXIE_CATHODE_NUM_PORT = DIGIT_SUPPRESS;
			NIXIE_CATHODE_L_DOT = 0;
			NIXIE_CATHODE_R_DOT = 0;
			wait_us(50);
		}
	}
}

// check hvps_status flag
void check_hvps(void){
	if(global_hvps_status == HVPS_ON || global_temp_turnon > 0 || global_do_shuffle > FALSE){
		on_hvps();
	}else{
		off_hvps();
	}
}

// generate all sec
void gen_all_sec(void){
	uint8_t temp_hour;

	// only calc when enabled shuffle & turnoff
	if(global_config_data.use_shuffle == ENABLE || global_config_data.use_turnoff == ENABLE){
		// sec
		global_now_all_sec = ((uint32_t)global_rtc_sec >> 4) * 10 + ((uint32_t)global_rtc_sec & 0b00001111);
		// min
		global_now_all_sec += (((uint32_t)global_rtc_min >> 4) * 10 + ((uint32_t)global_rtc_min & 0b00001111)) * 60;
		// hour
		// 12hr
		if(global_rtc_hour & RTC_AMPM_FLAG){
			temp_hour = global_rtc_hour;
			// 12hr PM 12-01-02-03-04-05-06-07-08-09-10-11
			if(temp_hour & RTC_PM_FLAG){
				temp_hour &= (uint8_t)~RTC_AMPM_FLAG;
				temp_hour &= (uint8_t)~RTC_PM_FLAG;
				// 12pm
				if(temp_hour == 0b00010010){
					global_now_all_sec += (((uint32_t)temp_hour >> 4) * 10 + ((uint32_t)temp_hour & 0b00001111)) * 3600;
				// 1-11pm
				}else{
					global_now_all_sec += (((uint32_t)temp_hour >> 4) * 10 + ((uint32_t)temp_hour & 0b00001111) + 12) * 3600;
				}
			// 12hr AM 12-01-02-03-04-05-06-07-08-09-10-11
			}else{
				temp_hour &= (uint8_t)~RTC_AMPM_FLAG;
				if(temp_hour != 0b00010010){ // 12hr AM 12 == 0
					global_now_all_sec += (((uint32_t)temp_hour >> 4) * 10 + ((uint32_t)temp_hour & 0b00001111)) * 3600;
				}
			}
		// 24hr
		}else{
			global_now_all_sec += (((uint32_t)global_rtc_hour >> 4) * 10 + ((uint32_t)global_rtc_hour & 0b00001111)) * 3600;
		}
	}
}

// check turnoff
void check_turnoff(void){
	// start_allsec or end_all_sec is calced at init or update
	// global_turnoff_all_sec[0] start
	// global_turnoff_all_sec[1] end
	if(global_config_data.use_turnoff == ENABLE){
		// start sec > end sec
		if(global_turnoff_all_sec[0] > global_turnoff_all_sec[1]){
			if(global_now_all_sec > global_turnoff_all_sec[0] || global_now_all_sec < global_turnoff_all_sec[1]){
				global_hvps_status &= (uint8_t)~HVPS_TIME;
			}else{
				global_hvps_status |= HVPS_TIME;
			}
		// start sec < end sec
		}else if(global_turnoff_all_sec[0] < global_turnoff_all_sec[1]){
			if(global_now_all_sec > global_turnoff_all_sec[0] && global_now_all_sec < global_turnoff_all_sec[1]){
				global_hvps_status &= (uint8_t)~HVPS_TIME;
			}else{
				global_hvps_status |= HVPS_TIME;
			}
		}else{
			global_hvps_status |= HVPS_TIME;
		}
	}else{
		global_hvps_status |= HVPS_TIME;
	}
}

// check shuffle time
void check_shuffle_time(void){
	if(global_do_shuffle == FALSE && global_config_data.use_shuffle == TRUE){
		if(global_now_all_sec == global_shuffle_all_sec[0] || global_now_all_sec == global_shuffle_all_sec[1]){
			do_shuffle();
		}
	}
}

// xor shift random
// 0 to 2^8 - 1
uint8_t rand_xor_shift(void){
	static uint8_t rand = XOR_SHIFT_SEED;
	static uint8_t count = 0;
	static uint8_t num = 0;
	
	rand ^= rand >> global_xor_shift_const[num][0];
	rand ^= rand << global_xor_shift_const[num][1];
	rand ^= rand >> global_xor_shift_const[num][2];

	// change const
	count++;
	if(count > 250){num++;}
	if(num == 24){
		num = 0;
	}

	return rand;
}

// chage test mode pattern
void change_test_pattern(void){
	switch(global_test_pattern){
	case RANDOM:
		global_test_pattern = PATTERN;
		break;

	case PATTERN:
		global_test_pattern = MANUAL;
		break;

	case MANUAL:
		global_test_pattern = RANDOM;
		break;

	default:
		global_test_pattern = RANDOM;
		break;
	}
}

// do shuffle
void do_shuffle(void){
	global_do_shuffle = ENABLE;
}

// stop shuffle
void stop_shuffle(void){
	global_do_shuffle = DISABLE;
}

// enable loop
void start_loop_shuffle(void){
	global_shuffle_continue = ENABLE;
}

// stop loop
void stop_loop_shuffle(void){
	global_shuffle_continue = DISABLE;
}

// do shuffle pattern
void do_shuffle_preset(void){
	global_shuffle_preset = ENABLE;
	global_do_shuffle = ENABLE;
}


// stop shuffle pattern
void stop_shuffle_preset(void){
	global_shuffle_preset = DISABLE;
	global_do_shuffle = DISABLE;
}

// leap year
uint8_t check_leap_year(uint8_t century, uint8_t year){
	uint16_t all_year;

	all_year = (uint16_t)century * 100 + (uint16_t)year;

	if(((all_year % 4) == 0 && (all_year % 100) != 0) || (all_year % 400) == 0){
		return 29;
	}
	return 28;
}

// month end
uint8_t get_monthend(uint8_t century, uint8_t year, uint8_t month){
	switch(month){
	case 1:
		return 31;

	case 2:
		return check_leap_year(century, year);

	case 3:
		return 31;

	case 4:
		return 30;

	case 5:
		return 31;

	case 6:
		return 30;

	case 7:
		return 31;

	case 8:
		return 31;

	case 9:
		return 30;

	case 10:
		return 31;

	case 11:
		return 30;

	case 12:
		return 31;

	default:
		return 31;
	}

	return 31;
}

// Zeller's congruence
uint8_t get_zeller_day(uint8_t century, uint8_t year, uint8_t month, uint8_t date){
	uint16_t day = 0;
	uint16_t all_year = 0;
	
	all_year  = (uint16_t)century * 100 + (uint16_t)year;
	if(month < 3){
		all_year--;
		month+=12;
	}
	day = (all_year + (uint16_t)(all_year / 4) - (uint16_t)(all_year / 100) + (uint16_t)(all_year / 400) + (uint16_t)((13 * (uint16_t)month + 8) / 5) + (uint16_t)date) % 7;
	// this formula gives 0-6, ds3232s needs 1-7
	day++;

	return day;
}
