#ifndef __CONFIG_H
#define __CONFIG_H

// # of digits
#define DIGIT_NUM 8

// Nixie duty
#define DUTY_CYCLE 50
#define DUTY_MAX 40
#define DUTY_MIN 10
#define DIGIT_SUPPRESS 0b00001010 // == 10

// Nixie anode
#define NIXIE_ANODE_PORT PORTA

// Nixie cathode num
#define NIXIE_CATHODE_NUM_PORT PORTC
#define NIXIE_CATHODE_MASK 0b11110000

// Nixie cathode dot
//#define NIXIE_CATHODE_R_DOT OCR1A
//#define NIXIE_CATHODE_L_DOT OCR1B
#define NIXIE_CATHODE_R_DOT OCR1B
#define NIXIE_CATHODE_L_DOT OCR1A


// Light sensor
#define LIGHT_SENSOR_PIN 7
#define LIGHT_SENSOR_TH_MAX 900
#define LIGHT_SENSOR_TH_MIN 0

// ALARM LED
#define ALARM_LED_PORT PORTF
#define ALARM_LED      PF3

// POWER LED
#define POWER_LED_PORT PORTF
#define POWER_LED      PF4

// Xbee uart LED
#define TX_LED_PORT PORTF
#define TX_LED PF5
#define RX_LED_PORT PORTF
#define RX_LED PF6

// GPS status LED
#define GPS_LED_PORT PORTG
#define GPS_LED PG0

// GPS power
#define GPS_PS_PORT PORTE
#define GPS_PS      PE2

// HVPS
#define HVPS_SW_PORT PORTG
#define HVPS_SW      PG2

// Encoder
#define ENC_A_PIN  PF0
#define ENC_SW_PIN PF1
#define ENC_B_PIN  PF2
#define ENC_PORT   PINF

// Long push
#define LONG_PUSH_COUNT 500

// mode
#define MODE_TIME        0
#define MODE_DATE        1
#define MODE_PRESSURE    2
#define MODE_TEMPERATURE 3
#define MODE_SHUFFLE     4
#define MODE_PHOTO       5
#define MODE_TEST        6

// config mode
#define CONFIG_NONE 0

/* Boolean type */
typedef enum{FALSE = 0, TRUE = 1} BOOL;
typedef enum{DISABLE = 0, ENABLE = 1} SWITCH;
typedef enum{FULL = 0, HALF = 1} AMPM;
typedef enum{UP = 0, DOWN = 1} PWM_DIRECTION;
typedef enum{RANDOM = 0, PATTERN = 1, MANUAL = 2} TEST_PATTERN;
typedef enum{BOTH = 0, RIGHT = 1, LEFT = 2, NONE = 3} TEST_PATTERN_DOT;
typedef enum{CEL = 0, FAH = 1, KEL = 2} TEMP_TYPE;
typedef enum{PA = 0, MMHG = 1} PRES_TYPE;

// dot mode
#define DOT_PWM   0
#define DOT_BLINK 1
#define DOT_ON    2
#define DOT_OFF   3
#define DOT_END   4

// dot PWM
#define DOT_PWM_MAX 250
#define DOT_PWM_MIN 4

// cross fade
#define XFADE_CYCLE   9000
#define XFADE_DIV     180
#define XFADE_DIV_MIN 50
#define XFADE_DIV_MAX 300
#define XFADE_DIV_SEN 75
#define XFADE_DIM     200
#define XFADE_NUM     16
#define XFADE_PATTERN_0  0b1000000000000000
#define XFADE_PATTERN_1  0b1000000010000000
#define XFADE_PATTERN_2  0b1000010000100000
#define XFADE_PATTERN_3  0b1000100010001000
#define XFADE_PATTERN_4  0b1001001001001000
#define XFADE_PATTERN_5  0b1001010010010100
#define XFADE_PATTERN_6  0b1010100101010100
#define XFADE_PATTERN_7  0b1010101010101010
#define XFADE_PATTERN_8  0b1010101110101010
#define XFADE_PATTERN_9  0b1010111010111010
#define XFADE_PATTERN_10 0b1110111011101100
#define XFADE_PATTERN_11 0b1110111011101110
#define XFADE_PATTERN_12 0b1111101111011110
#define XFADE_PATTERN_13 0b0111111101111111
#define XFADE_PATTERN_14 0b1111111111111110
#define XFADE_PATTERN_15 0b1111111111111111
#define XFADE_DIV_NOW global_config_eep_map.xfade_div

// current mode
#define CURRENT_MODE global_config_data.mode

// shuffle mode
#define SHUFFLE_MODE_1 1
#define SHUFFLE_MODE_2 2
#define SHUFFLE_MODE_3 3
#define SHUFFLE_MODE_4 4
#define SHUFFLE_MODE_5 5
#define SHUFFLE_MODE_6 6
#define SHUFFLE_END    7
#define SHUFFLE_MODE_CHANGE 1000
#define STOP_COUNT 100
#define SCROLL_COUNT 12

// operation time
#define SEC_UPDATE_COUNT     2*2*60*60            // by 2hour(2clk/sec)
#define SEC_UPDATE_INCREMENT (SEC_UPDATE_COUNT/2)

// HVPS status
#define HVPS_MAIN   0b00000001
#define HVPS_SENSOR 0b00000010
#define HVPS_TIME   0b00000100
#define HVPS_ON (HVPS_MAIN | HVPS_SENSOR | HVPS_TIME)

// for xor shift seed
#define XOR_SHIFT_SEED 39 // mikudayo-

#define CONFIG_TIME_NONE 0
#define CONFIG_TIME_HOUR 1
#define CONFIG_TIME_MIN  2
#define CONFIG_TIME_SEC  3

#define CONFIG_DATE_NONE    0
#define CONFIG_DATE_CENTURY 1
#define CONFIG_DATE_YEAR    2
#define CONFIG_DATE_MONTH   3
#define CONFIG_DATE_DATE    4

#define CONFIG_PHOTO_DIGIT_NONE 0
#define CONFIG_PHOTO_DIGIT_1    1   // DIGIT 1
#define CONFIG_PHOTO_DIGIT_2    2   // DOT 1
#define CONFIG_PHOTO_DIGIT_3    3   // DIGIT 2
#define CONFIG_PHOTO_DIGIT_4    4   // DOT 2 ...
#define CONFIG_PHOTO_DIGIT_5    5
#define CONFIG_PHOTO_DIGIT_6    6
#define CONFIG_PHOTO_DIGIT_7    7
#define CONFIG_PHOTO_DIGIT_8    8
#define CONFIG_PHOTO_DIGIT_9    9
#define CONFIG_PHOTO_DIGIT_10   10
#define CONFIG_PHOTO_DIGIT_11   11
#define CONFIG_PHOTO_DIGIT_12   12
#define CONFIG_PHOTO_DIGIT_13   13
#define CONFIG_PHOTO_DIGIT_14   14
#define CONFIG_PHOTO_DIGIT_15   15
#define CONFIG_PHOTO_DIGIT_16   16

#endif
