#include <avr/eeprom.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>

#include "xitoa.h"
#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"


void eep_write_config(void){
	eeprom_busy_wait();
	eeprom_write_block(&global_config_data, &global_config_eep_map, sizeof(CONFIG_DATA));
}

void eep_read_config(void){
	eeprom_busy_wait();
	eeprom_read_block(&global_config_data, &global_config_eep_map, sizeof(CONFIG_DATA));
}

// read setting & pressure data from eeprom
void read_eeprom_setting(void){
	uint8_t temp;
	uint8_t i;

	// read config
	eep_read_config();

	// check config data
	// mode
	if(global_config_data.mode < 0 || global_config_data.mode > MODE_TEST){
		eep_update_mode(E_MODE_DEFAULT);
	}

	// century upper 4bit is 10 century, lower 4bit is 1 century
	temp = 0b00001111 & global_config_data.century;
	if(temp > 9){
		eep_update_century(E_CENTURY_DEFAULT);
	}
	temp = (0b11110000 & global_config_data.century) >> 4;
	if(temp > 9){
		eep_update_century(E_CENTURY_DEFAULT);
	}

	// ampm
	if(global_config_data.ampm < FULL || global_config_data.ampm > HALF){
		eep_update_ampm(E_AMPM_DEFAULT);
	}

	// duty
	if(global_config_data.duty < DUTY_MIN || global_config_data.duty > DUTY_MAX){
		eep_update_duty(E_NIXIE_DUTY_DEFAULT);
	}

	// dot mode
	if(global_config_data.dot_mode < 0 || global_config_data.dot_mode >= DOT_END){
		eep_update_dot_mode(E_DOT_MODE_DEFAULT);
	}

	// sec count

	// use shuffle
	if(global_config_data.use_shuffle < DISABLE || global_config_data.use_shuffle > ENABLE){
		eep_update_use_shuffle(E_USE_SHUFFLE_DEFAULT);
	}

	// shuffle hour min sec
	// hour
	if(global_config_data.shuffle_hour[0] < 0 || global_config_data.shuffle_hour[0] > 24){
		global_config_data.shuffle_hour[0] = E_SHUFFLE_HOUR0_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_hour[0], E_SHUFFLE_HOUR0_DEFAULT);
	}
	if(global_config_data.shuffle_hour[1] < 0 || global_config_data.shuffle_hour[1] > 24){
		global_config_data.shuffle_hour[1] = E_SHUFFLE_HOUR1_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_hour[1], E_SHUFFLE_HOUR1_DEFAULT);
	}
	// min
	if(global_config_data.shuffle_min[0] < 0 || global_config_data.shuffle_min[0] > 59){
		global_config_data.shuffle_min[0] = E_SHUFFLE_MIN0_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_min[0], E_SHUFFLE_MIN0_DEFAULT);
	}
	if(global_config_data.shuffle_min[1] < 0 || global_config_data.shuffle_min[1] > 59){
		global_config_data.shuffle_min[1] = E_SHUFFLE_MIN1_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_min[1], E_SHUFFLE_MIN1_DEFAULT);
	}
	// sec
	if(global_config_data.shuffle_sec[0] < 0 || global_config_data.shuffle_sec[0] > 59){
		global_config_data.shuffle_sec[0] = E_SHUFFLE_SEC0_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_sec[0], E_SHUFFLE_SEC0_DEFAULT);
	}
	if(global_config_data.shuffle_sec[1] < 0 || global_config_data.shuffle_sec[1] > 59){
		global_config_data.shuffle_sec[1] = E_SHUFFLE_SEC1_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_sec[1], E_SHUFFLE_SEC1_DEFAULT);
	}
	global_shuffle_all_sec[0] = (uint32_t)global_config_data.shuffle_hour[0] * 3600 + (uint32_t)global_config_data.shuffle_min[0] * 60 + (uint32_t)global_config_data.shuffle_sec[0];
	global_shuffle_all_sec[1] = (uint32_t)global_config_data.shuffle_hour[1] * 3600 + (uint32_t)global_config_data.shuffle_min[1] * 60 + (uint32_t)global_config_data.shuffle_sec[1];

	// use cross fade
	if(global_config_data.use_xfade < DISABLE || global_config_data.use_xfade > ENABLE){
		eep_update_use_xfade(E_USE_XFADE_DEFAULT);
	}

	// cross fade div
	if(global_config_data.xfade_div < XFADE_DIV_MIN || global_config_data.xfade_div > XFADE_DIV_MAX){
		eep_update_xfade_div(E_XFADE_DIV_DEFAULT);
	}
	global_xfade_div_temp = global_config_data.xfade_div;
	global_xfade_edge = XFADE_CYCLE - global_config_data.xfade_div * XFADE_NUM - XFADE_DIM * (DUTY_MAX - global_config_data.duty);
	for(i=0; i<XFADE_NUM; i++){
		global_xfade_term[i] = global_xfade_edge + global_config_data.xfade_div * (i + 1);
	}

	// use light sensor
	if(global_config_data.use_light_sensor < DISABLE || global_config_data.use_light_sensor > ENABLE){
		eep_update_use_light_sensor(E_USE_LIGHT_SENSOR_DEFAULT);
	}

	// light sensor threshold
	if(global_config_data.light_sensor_th < LIGHT_SENSOR_TH_MIN || global_config_data.light_sensor_th > LIGHT_SENSOR_TH_MAX){
		eep_update_light_sensor_th(E_LIGHT_SENSOR_TH_DEFAULT);
	}

	// light sensor hvps
	if(global_config_data.light_sensor_hvps < DISABLE || global_config_data.light_sensor_hvps > ENABLE){
		eep_update_light_sensor_hvps(E_LIGHT_SENSOR_HVPS_DEFAULT);
	}

	// use turnoff
	if(global_config_data.use_turnoff < DISABLE || global_config_data.use_turnoff > ENABLE){
		eep_update_use_turnoff(E_USE_TURNOFF_DEFAULT);
	}

	// turnoff hour min sec
	// hour
	if(global_config_data.turnoff_hour[0] < 0 || global_config_data.turnoff_hour[0] > 24){
		global_config_data.turnoff_hour[0] = E_TURNOFF_START_HOUR_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_hour[0], E_TURNOFF_START_HOUR_DEFAULT);
	}
	if(global_config_data.turnoff_hour[1] < 0 || global_config_data.turnoff_hour[1] > 24){
		global_config_data.turnoff_hour[1] = E_TURNOFF_END_HOUR_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_hour[1], E_TURNOFF_END_HOUR_DEFAULT);
	}
	// min
	if(global_config_data.turnoff_min[0] < 0 || global_config_data.turnoff_min[0] > 59){
		global_config_data.turnoff_min[0] = E_TURNOFF_START_MIN_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_min[0], E_TURNOFF_START_MIN_DEFAULT);
	}
	if(global_config_data.turnoff_min[1] < 0 || global_config_data.turnoff_min[1] > 59){
		global_config_data.turnoff_min[1] = E_TURNOFF_END_MIN_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_min[1], E_TURNOFF_END_MIN_DEFAULT);
	}
	// sec
	if(global_config_data.turnoff_sec[0] < 0 || global_config_data.turnoff_sec[0] > 59){
		global_config_data.turnoff_sec[0] = E_TURNOFF_START_SEC_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_sec[0], E_TURNOFF_START_SEC_DEFAULT);
	}
	if(global_config_data.turnoff_sec[1] < 0 || global_config_data.turnoff_sec[1] > 59){
		global_config_data.turnoff_sec[1] = E_TURNOFF_END_SEC_DEFAULT;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_sec[1], E_TURNOFF_END_SEC_DEFAULT);
	}
	global_turnoff_all_sec[0] = (uint32_t)global_config_data.turnoff_hour[0] * 3600 + (uint32_t)global_config_data.turnoff_min[0] * 60 + (uint32_t)global_config_data.turnoff_sec[0];
	global_turnoff_all_sec[1] = (uint32_t)global_config_data.turnoff_hour[1] * 3600 + (uint32_t)global_config_data.turnoff_min[1] * 60 + (uint32_t)global_config_data.turnoff_sec[1];

	// use gps
	if(global_config_data.use_gps < DISABLE || global_config_data.use_gps > ENABLE){
		eep_update_use_gps(E_USE_GPS_DEFAULT);
	}

	// gps log
	//global_config_data.gps_log = E_USE_GPS_LOG_DEFAULT;
	if(global_config_data.gps_log < DISABLE || global_config_data.gps_log > ENABLE){
		eep_update_gps_log(E_USE_GPS_LOG_DEFAULT);
	}

	// gps timezone
	if(global_config_data.gps_tz < GPS_TIMEZONE_MIN || global_config_data.gps_tz > GPS_TIMEZONE_MAX){
		eep_update_gps_tz(E_GPS_TIMEZONE_DEFAULT);
	}

	// shuffle
	for(i=0; i<DIGIT_NUM; i++){
		if(global_config_data.shuffle_digit[i] < 0 || global_config_data.shuffle_digit[i] > 9){
			global_config_data.shuffle_digit[i] = E_SHUFFLE_DIGIT_DEFAULT;
			eeprom_busy_wait();
			eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_digit[i], E_SHUFFLE_DIGIT_DEFAULT);
		}
	}

	// photo
	// num
	for(i=0; i<DIGIT_NUM; i++){
		if(global_config_data.photo_digit[i] < 0 || global_config_data.photo_digit[i] > DIGIT_SUPPRESS){
			global_config_data.photo_digit[i] = E_PHOTO_DIGIT_DEFAULT;
			eeprom_busy_wait();
			eeprom_write_byte((uint8_t *)&global_config_eep_map.photo_digit[i], E_PHOTO_DIGIT_DEFAULT);
		}
	}
	// photo dot allow all num

	// temp type
	if(global_config_data.temp_type < CEL || global_config_data.temp_type > KEL){
		eep_update_temp_type(E_TEMP_TYPE_DEFAULT);
	}

	// pressure type
	if(global_config_data.pres_type < PA || global_config_data.pres_type > MMHG){
		eep_update_pres_type(E_PRES_TYPE_DEFAULT);
	}
}

void reset_eeprom_setting(void){
	uint8_t i;
	uint8_t century[2];

	century[0] = E_CENTURY_DEFAULT / 10;
	century[1] = E_CENTURY_DEFAULT - century[0] * 10;

	// setting
	global_config_data.mode = E_MODE_DEFAULT;
	global_config_data.century = century[0] << 4 | century[1];
	global_config_data.ampm = E_AMPM_DEFAULT;
	global_config_data.duty = E_NIXIE_DUTY_DEFAULT;
	global_config_data.dot_mode = E_DOT_MODE_DEFAULT;
	global_config_data.sec_count = E_SEC_COUNT_DEFAULT;
	global_config_data.use_shuffle = E_USE_SHUFFLE_DEFAULT;
	global_config_data.shuffle_hour[0] = E_SHUFFLE_HOUR0_DEFAULT;
	global_config_data.shuffle_hour[1] = E_SHUFFLE_HOUR1_DEFAULT;
	global_config_data.shuffle_min[0] = E_SHUFFLE_MIN0_DEFAULT;
	global_config_data.shuffle_min[1] = E_SHUFFLE_MIN1_DEFAULT;
	global_config_data.shuffle_sec[0] = E_SHUFFLE_SEC0_DEFAULT;
	global_config_data.shuffle_sec[1] = E_SHUFFLE_SEC1_DEFAULT;
	global_config_data.use_xfade = E_USE_XFADE_DEFAULT;
	global_config_data.xfade_div = E_XFADE_DIV_DEFAULT;
	global_xfade_div_temp = global_config_data.xfade_div;
	global_xfade_edge = XFADE_CYCLE - global_config_data.xfade_div * XFADE_NUM - XFADE_DIM * (DUTY_MAX - global_config_data.duty);
	for(i=0; i<XFADE_NUM; i++){
		global_xfade_term[i] = global_xfade_edge + global_config_data.xfade_div * (i + 1);
	}
	global_config_data.use_light_sensor = E_USE_LIGHT_SENSOR_DEFAULT;
	global_config_data.light_sensor_th = E_LIGHT_SENSOR_TH_DEFAULT;
	global_config_data.light_sensor_hvps = E_LIGHT_SENSOR_HVPS_DEFAULT;
	global_config_data.use_turnoff = E_USE_TURNOFF_DEFAULT;
	global_config_data.turnoff_hour[0] = E_TURNOFF_START_HOUR_DEFAULT;
	global_config_data.turnoff_hour[1] = E_TURNOFF_END_HOUR_DEFAULT;
	global_config_data.turnoff_min[0] = E_TURNOFF_START_MIN_DEFAULT;
	global_config_data.turnoff_min[1] = E_TURNOFF_END_MIN_DEFAULT;
	global_config_data.turnoff_sec[0] = E_TURNOFF_START_SEC_DEFAULT;
	global_config_data.turnoff_sec[1] = E_TURNOFF_END_SEC_DEFAULT;
	global_config_data.use_gps = E_USE_GPS_DEFAULT;
	global_config_data.gps_log = E_USE_GPS_LOG_DEFAULT;
	global_config_data.gps_tz = E_GPS_TIMEZONE_DEFAULT;
	for(i=0; i<DIGIT_NUM; i++){
		global_config_data.shuffle_digit[i] = E_SHUFFLE_DIGIT_DEFAULT;
		global_config_data.photo_digit[i] = E_PHOTO_DIGIT_DEFAULT;
		global_config_data.photo_L_dot[i] = E_PHOTO_DOT_DEFAULT;
		global_config_data.photo_R_dot[i] = E_PHOTO_DOT_DEFAULT;
	}
	global_config_data.temp_type = E_TEMP_TYPE_DEFAULT;
	global_config_data.pres_type = E_PRES_TYPE_DEFAULT;
	eep_write_config();

	global_shuffle_all_sec[0] = (uint32_t)global_config_data.shuffle_hour[0] * 3600 + (uint32_t)global_config_data.shuffle_min[0] * 60 + (uint32_t)global_config_data.shuffle_sec[0];
	global_shuffle_all_sec[1] = (uint32_t)global_config_data.shuffle_hour[1] * 3600 + (uint32_t)global_config_data.shuffle_min[1] * 60 + (uint32_t)global_config_data.shuffle_sec[1];
	global_turnoff_all_sec[0] = (uint32_t)global_config_data.turnoff_hour[0] * 3600 + (uint32_t)global_config_data.turnoff_min[0] * 60 + (uint32_t)global_config_data.turnoff_sec[0];
	global_turnoff_all_sec[1] = (uint32_t)global_config_data.turnoff_hour[1] * 3600 + (uint32_t)global_config_data.turnoff_min[1] * 60 + (uint32_t)global_config_data.turnoff_sec[1];
}


void eep_update_mode(uint8_t mode){
	if(mode >= 0 && mode <= MODE_TEST){
		global_config_data.mode = mode;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.mode, mode);
	}
}

void eep_update_century(uint8_t century){
	uint8_t temp[3];
	
	if(century > 99){return;}
	sprintf((char *)temp, "%02d", century);
	temp[0] -= 48;
	temp[1] -= 48;
	temp[0] = temp[0] << 4;
	global_config_data.century = temp[0] | temp[1];
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.century,  temp[0] | temp[1]);
}

void eep_update_ampm(uint8_t ampm){
	if(ampm >= FULL && ampm <= HALF){
		global_config_data.ampm = ampm;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.ampm, ampm);
	}
}

void eep_update_duty(uint8_t duty){
	uint8_t i;

	if(duty >= DUTY_MIN && duty <= DUTY_MAX){
		global_config_data.duty = duty;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.duty, duty);
		for(i=0; i<DIGIT_NUM; i++){
			global_digit_duty[i] = duty;
		}
		// update cfade div
		eep_update_xfade_div(global_config_data.xfade_div);
	}
}

void toggle_dot_mode(void){
	global_config_data.dot_mode++;
	if(global_config_data.dot_mode == DOT_END){global_config_data.dot_mode = DOT_PWM;}
	eep_update_dot_mode(global_config_data.dot_mode);
}

void eep_update_dot_mode(uint8_t dot_mode){
	if(dot_mode >= 0 && dot_mode < DOT_END){
		global_config_data.dot_mode = dot_mode;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.dot_mode, dot_mode);
	}
}

void eep_update_sec_count(uint32_t sec_count){
	global_config_data.sec_count = sec_count;
	eeprom_busy_wait();
	eeprom_update_dword((uint32_t *)&global_config_eep_map.sec_count, sec_count);
}

void eep_update_use_shuffle(uint8_t use_shuffle){
	if(use_shuffle >= DISABLE && use_shuffle <= ENABLE){
		global_config_data.use_shuffle = use_shuffle;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_shuffle, use_shuffle);
	}
}

void eep_update_shuffle_time_1(uint8_t hour, uint8_t min, uint8_t sec){
	// check
	if(hour < 0 || hour > 24){return;}
	if(min < 0 || min > 59){return;}
	if(sec < 0 || sec > 59){return;}

	// hour
	global_config_data.shuffle_hour[0] = hour;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_hour[0], hour);
	// min
	global_config_data.shuffle_min[0] = min;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_min[0], min);
	// sec
	global_config_data.shuffle_sec[0] = sec;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_sec[0], sec);

	global_shuffle_all_sec[0] = (uint32_t)hour * 3600 + (uint32_t)min * 60 + (uint32_t)sec;
}

void eep_update_shuffle_time_2(uint8_t hour, uint8_t min, uint8_t sec){
	// check
	if(hour < 0 || hour > 24){return;}
	if(min < 0 || min > 59){return;}
	if(sec < 0 || sec > 59){return;}

	// hour
	global_config_data.shuffle_hour[1] = hour;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_hour[1], hour);
	// min
	global_config_data.shuffle_min[1] = min;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_min[1], min);
	// sec
	global_config_data.shuffle_sec[1] = sec;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.shuffle_sec[1], sec);

	global_shuffle_all_sec[1] = (uint32_t)hour * 3600 + (uint32_t)min * 60 + (uint32_t)sec;
}

void eep_update_use_xfade(uint8_t use_xfade){
	if(use_xfade >= DISABLE && use_xfade <= ENABLE){
		global_config_data.use_xfade = use_xfade;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_xfade, use_xfade);
	}

}

void eep_update_xfade_div(uint8_t xfade_div){
	uint8_t i;

	if(xfade_div >= XFADE_DIV_MIN && xfade_div <= XFADE_DIV_MAX){
		global_config_data.xfade_div = xfade_div;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.xfade_div, xfade_div);
	}
	global_xfade_div_temp = global_config_data.xfade_div;
	global_xfade_edge = XFADE_CYCLE - global_config_data.xfade_div * XFADE_NUM - XFADE_DIM * (DUTY_MAX - global_config_data.duty);
	for(i=0; i<XFADE_NUM; i++){
		global_xfade_term[i] = global_xfade_edge + global_config_data.xfade_div * (i + 1);
	}
}

void eep_update_use_light_sensor(uint8_t use_light_sensor){
	if(use_light_sensor >= DISABLE && use_light_sensor <= ENABLE){
		global_config_data.use_light_sensor = use_light_sensor;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_light_sensor, use_light_sensor);
	}
	normal_duty();
}

void eep_update_light_sensor_th(uint16_t light_sensor_th){
	if(light_sensor_th >= LIGHT_SENSOR_TH_MIN && light_sensor_th <= LIGHT_SENSOR_TH_MAX){
		global_config_data.light_sensor_th = light_sensor_th;
		eeprom_busy_wait();
		eeprom_write_word((uint16_t *)&global_config_eep_map.light_sensor_th, light_sensor_th);
	}
}

void eep_update_light_sensor_hvps(uint8_t light_sensor_hvps){
	if(light_sensor_hvps >= DISABLE && light_sensor_hvps <= ENABLE){
		global_config_data.light_sensor_hvps = light_sensor_hvps;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.light_sensor_hvps, light_sensor_hvps);
	}
}

void eep_update_use_turnoff(uint8_t use_turnoff){
	if(use_turnoff >= DISABLE && use_turnoff <= ENABLE){
		global_config_data.use_turnoff = use_turnoff;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_turnoff, use_turnoff);
	}
}

void eep_update_turnoff_start(uint8_t hour, uint8_t min, uint8_t sec){
	// check
	if(hour < 0 || hour > 24){return;}
	if(min < 0 || min > 59){return;}
	if(sec < 0 || sec > 5){return;}

	// hour	
	global_config_data.turnoff_hour[0] = hour;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_hour[0], hour);
	// min
	global_config_data.turnoff_min[0] = min;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_min[0], min);
	// sec
	global_config_data.turnoff_sec[0] = sec;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_sec[0], sec);

	global_turnoff_all_sec[0] = (uint32_t)hour * 3600 + (uint32_t)min * 60 + (uint32_t)sec;
}

void eep_update_turnoff_end(uint8_t hour, uint8_t min, uint8_t sec){
	// check
	if(hour < 0 || hour > 24){return;}
	if(min < 0 || min > 59){return;}
	if(sec < 0 || sec > 5){return;}

	// hour
	global_config_data.turnoff_hour[1] = hour;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_hour[1], hour);
	// min
	global_config_data.turnoff_min[1] = min;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_min[1], min);
	// sec
	global_config_data.turnoff_sec[1] = sec;
	eeprom_busy_wait();
	eeprom_write_byte((uint8_t *)&global_config_eep_map.turnoff_sec[1], sec);

	global_turnoff_all_sec[1] = (uint32_t)hour * 3600 + (uint32_t)min * 60 + (uint32_t)sec;
}

void eep_update_use_gps(uint8_t use_gps){
	if(use_gps >= DISABLE && global_config_data.use_gps <= ENABLE){
		global_config_data.use_gps = use_gps;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.use_gps, use_gps);
	}
}

void eep_update_gps_log(uint8_t gps_log){
	if(gps_log >= DISABLE && gps_log <= ENABLE){
		global_config_data.gps_log = gps_log;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.gps_log, gps_log);
	}
}

void eep_update_gps_tz(int8_t gps_tz){
	if(gps_tz >= GPS_TIMEZONE_MIN && gps_tz <= GPS_TIMEZONE_MAX){
		global_config_data.gps_tz = gps_tz;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.gps_tz, gps_tz);
	}
}


// digit is 7 elements
void eep_update_shuffle_digit(uint8_t *digit, uint8_t digit_size){
	uint8_t i;
	uint8_t temp_digit[DIGIT_NUM];

	if(digit_size != DIGIT_NUM - 1){return;}
	for(i=0; i<DIGIT_NUM; i++){
		if(digit[i] < 0 || digit[i] > 9){return;}
	}

	temp_digit[0] = digit[0];
	temp_digit[1] = DIGIT_SUPPRESS;
	temp_digit[2] = digit[1];
	temp_digit[3] = digit[2];
	temp_digit[4] = digit[3];
	temp_digit[5] = digit[4];
	temp_digit[6] = digit[5];
	temp_digit[7] = digit[6];

	global_config_data.shuffle_digit[0] = temp_digit[0];
	global_config_data.shuffle_digit[1] = temp_digit[1];
	global_config_data.shuffle_digit[2] = temp_digit[2];
	global_config_data.shuffle_digit[3] = temp_digit[3];
	global_config_data.shuffle_digit[4] = temp_digit[4];
	global_config_data.shuffle_digit[5] = temp_digit[5];
	global_config_data.shuffle_digit[6] = temp_digit[6];
	global_config_data.shuffle_digit[7] = temp_digit[7];

	eeprom_busy_wait();
	eeprom_write_block(temp_digit, &global_config_eep_map.shuffle_digit, sizeof(temp_digit));
}

void eep_update_photo_digit(uint8_t *digit, uint8_t digit_size, uint8_t *L_dot, uint8_t L_dot_size, uint8_t *R_dot, uint8_t R_dot_size){
	uint8_t i;

	if(digit_size != DIGIT_NUM){return;}
	if(L_dot_size != DIGIT_NUM){return;}
	if(R_dot_size != DIGIT_NUM){return;}

	for(i=0; i<DIGIT_NUM; i++){
		if(digit[i] < 0 || digit[i] > DIGIT_SUPPRESS){return;}
		if(L_dot[i] != 0){L_dot[i] = DOT_PWM_MAX;}
		if(R_dot[i] != 0){R_dot[i] = DOT_PWM_MAX;}
	}

	for(i=0; i<DIGIT_NUM; i++){
		global_config_data.photo_digit[i] = digit[i];
		global_config_data.photo_L_dot[i] = L_dot[i];
		global_config_data.photo_R_dot[i] = R_dot[i];
	}

	eeprom_busy_wait();
	eeprom_write_block(digit, &global_config_eep_map.photo_digit, digit_size);
	eeprom_busy_wait();
	eeprom_write_block(L_dot, &global_config_eep_map.photo_L_dot, L_dot_size);
	eeprom_busy_wait();
	eeprom_write_block(R_dot, &global_config_eep_map.photo_R_dot, R_dot_size);
}

void eep_update_serial(uint32_t serial){
	global_config_data.serial = serial;
	eeprom_busy_wait();
	eeprom_update_dword((uint32_t *)&global_config_eep_map.serial, serial);
}

void eep_update_temp_type(uint8_t type){
	if(type >= CEL && type <= KEL){
		global_config_data.temp_type = type;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.temp_type, type);
	}
}

void eep_update_pres_type(uint8_t type){
	if(type >= PA && type <= MMHG){
		global_config_data.pres_type = type;
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)&global_config_eep_map.pres_type, type);
	}
}
