#ifndef __EEPROM_H
#define __EEPROM_H

#include "config.h"

typedef struct _config_data{
	volatile uint8_t mode;
	volatile uint8_t century;
	volatile uint8_t ampm;
	volatile uint8_t duty;
	volatile uint8_t dot_mode;
	volatile uint32_t sec_count;
	volatile uint8_t use_shuffle;
	volatile uint8_t shuffle_hour[2];
	volatile uint8_t shuffle_min[2];
	volatile uint8_t shuffle_sec[2];
	volatile uint8_t use_xfade;
	volatile uint16_t xfade_div;
	volatile uint8_t use_light_sensor;
	volatile uint16_t light_sensor_th;
	volatile uint8_t light_sensor_hvps;
	volatile uint8_t use_turnoff;
	volatile uint8_t turnoff_hour[2];
	volatile uint8_t turnoff_min[2];
	volatile uint8_t turnoff_sec[2];
	volatile uint8_t use_gps;
	volatile uint8_t gps_log;
	volatile int8_t  gps_tz;
	volatile uint8_t shuffle_digit[DIGIT_NUM];
	volatile uint8_t photo_digit[DIGIT_NUM];
	volatile uint8_t photo_L_dot[DIGIT_NUM];
	volatile uint8_t photo_R_dot[DIGIT_NUM];
	volatile uint8_t temp_type;
	volatile uint8_t pres_type;
	volatile uint32_t serial;
}CONFIG_DATA;

#define E_MODE_DEFAULT               MODE_TIME
#define E_CENTURY_DEFAULT            20
#define E_AMPM_DEFAULT               FULL
#define E_NIXIE_DUTY_DEFAULT         40
#define E_DOT_MODE_DEFAULT           DOT_PWM
#define E_SEC_COUNT_DEFAULT          0
#define E_USE_SHUFFLE_DEFAULT        ENABLE
#define E_SHUFFLE_HOUR0_DEFAULT      23 // 23:30:00
#define E_SHUFFLE_MIN0_DEFAULT       30
#define E_SHUFFLE_SEC0_DEFAULT       0
#define E_SHUFFLE_HOUR1_DEFAULT      6 // 06:15:00
#define E_SHUFFLE_MIN1_DEFAULT       15
#define E_SHUFFLE_SEC1_DEFAULT       0
#define E_USE_XFADE_DEFAULT          ENABLE
#define E_XFADE_DIV_DEFAULT          XFADE_DIV
#define E_USE_LIGHT_SENSOR_DEFAULT   ENABLE
#define E_LIGHT_SENSOR_TH_DEFAULT    150
#define E_LIGHT_SENSOR_HVPS_DEFAULT  DISABLE
#define E_USE_TURNOFF_DEFAULT        DISABLE
#define E_TURNOFF_START_HOUR_DEFAULT 0 // from 00:00:00
#define E_TURNOFF_START_MIN_DEFAULT  0
#define E_TURNOFF_START_SEC_DEFAULT  0
#define E_TURNOFF_END_HOUR_DEFAULT   6 // to 06:00:00
#define E_TURNOFF_END_MIN_DEFAULT    0
#define E_TURNOFF_END_SEC_DEFAULT    0
#define E_USE_GPS_DEFAULT            ENABLE
#define E_USE_GPS_LOG_DEFAULT        DISABLE
#define E_GPS_TIMEZONE_DEFAULT       9 // japan
#define E_SHUFFLE_DIGIT_DEFAULT      0
#define E_PHOTO_DIGIT_DEFAULT        0
#define E_PHOTO_DOT_DEFAULT          0
#define E_TEMP_TYPE_DEFAULT          CEL
#define E_PRES_TYPE_DEFAULT          PA

void read_eeprom_setting(void);
void reset_eeprom_setting(void);
void eep_update_mode(uint8_t mode);
void eep_update_century(uint8_t century);
void eep_update_ampm(uint8_t ampm);
void eep_update_duty(uint8_t duty);
void toggle_dot_mode(void);
void eep_update_dot_mode(uint8_t dot_mode);
void eep_update_sec_count(uint32_t sec_count);
void eep_update_use_shuffle(uint8_t use_shuffle);
void eep_update_shuffle_time_1(uint8_t hour, uint8_t min, uint8_t sec);
void eep_update_shuffle_time_2(uint8_t hour, uint8_t min, uint8_t sec);
void eep_update_use_xfade(uint8_t use_xfade);
void eep_update_xfade_div(uint8_t xfade_div);
void eep_update_use_light_sensor(uint8_t use_light_sensor);
void eep_update_light_sensor_th(uint16_t light_sensor_th);
void eep_update_light_sensor_hvps(uint8_t light_sensor_hvps);
void eep_update_use_turnoff(uint8_t use_turnoff);
void eep_update_turnoff_start(uint8_t hour, uint8_t min, uint8_t sec);
void eep_update_turnoff_end(uint8_t hour, uint8_t min, uint8_t sec);
void eep_update_use_gps(uint8_t use_gps);
void eep_update_gps_log(uint8_t gps_log);
void eep_update_gps_tz(int8_t gps_tz);
void eep_update_shuffle_digit(uint8_t *digit, uint8_t digit_size);
void eep_update_photo_digit(uint8_t *digit, uint8_t digit_size, uint8_t *L_dot, uint8_t L_dot_size, uint8_t *R_dot, uint8_t R_dot_size);
void eep_update_serial(uint32_t serial);
void eep_update_temp_type(uint8_t type);
void eep_update_pres_type(uint8_t type);

#endif
