#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "LPS331.h"
#include "mode.h"
#include "xitoa.h"

void mode_time(void){
	uint8_t ampm_flag;
	uint8_t pm_flag;
	static uint8_t old_sec;

	// RTC data is updated @ ISR(INT7_vect)

	// dot up/down flag
	if(global_rtc_sec & 0b00000001){ // odd
		global_pwm_direction = DOWN;
	}else{ // even
		global_pwm_direction = UP;
	}

	// check 24hr/12hr
	if(global_rtc_hour & RTC_AMPM_FLAG){
		ampm_flag = ENABLE;
	}else{
		ampm_flag = DISABLE;
	}
	// check 12hr am/pm
	if(global_rtc_hour & RTC_PM_FLAG){
		pm_flag = ENABLE;
	}else{
		pm_flag = DISABLE;
	}
	
	// remove 12hr flag
	if(ampm_flag == ENABLE){
		global_digit_num[0] = (global_rtc_hour & 0b00010000) >> 4;
	}else{
		global_digit_num[0] = (global_rtc_hour & 0b00110000) >> 4;
	}
	global_digit_num[1] = global_rtc_hour & 0b00001111;
	global_digit_num[2] = DIGIT_SUPPRESS;
	global_digit_num[3] = global_rtc_min >> 4;
	global_digit_num[4] = global_rtc_min & 0b00001111;
	global_digit_num[5] = DIGIT_SUPPRESS;
	global_digit_num[6] = global_rtc_sec >> 4;
	global_digit_num[7] = global_rtc_sec & 0b00001111;

	global_digit_L_dot_pwm[0] = 0;
	global_digit_L_dot_pwm[1] = 0;
	global_digit_L_dot_pwm[2] = 0;
	global_digit_L_dot_pwm[3] = 0;
	global_digit_L_dot_pwm[4] = 0;
	global_digit_L_dot_pwm[5] = 0;
	global_digit_L_dot_pwm[6] = 0;
	global_digit_L_dot_pwm[7] = 0;

	// 12hr dot indicator
	if(ampm_flag == ENABLE && pm_flag == DISABLE){ // 12hr @ am
		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[1] = 0;
	}else if(ampm_flag == ENABLE && pm_flag == ENABLE){ // 12hr @ pm
		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;
	}else{ // 24hr
		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = 0;
	}
	//global_digit_R_dot_pwm[2] = DOT_PWM_MAX; // R dot 2 & 5 is controlled by timer2
	global_digit_R_dot_pwm[3] = 0;
	global_digit_R_dot_pwm[4] = 0;
	//global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
	global_digit_R_dot_pwm[6] = 0;
	// gps signal dot
	global_gps_count++;
	if(global_gps_count < GPS_COUNT){
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
	}else{
		global_digit_R_dot_pwm[7] = 0;
	}

#ifdef DEBUG_TIMING_2
	xprintf(PSTR("global_xfade_count=%d \n"), global_xfade_count);
	xprintf(PSTR("edge=%d \n"), global_xfade_edge);
#endif

	if(old_sec != global_digit_num[7]){
		old_sec = global_digit_num[7];
		global_xfade_count = 0;
	}
}

void mode_date(void){
	// RTC data is updated @ ISR(INT7_vect)

	if(global_rtc_sec & 0b00000001){ // odd
		global_pwm_direction = DOWN;
	}else{ // even
		global_pwm_direction = UP;
	}

	global_digit_num[0] = global_config_data.century >> 4;
	global_digit_num[1] = global_config_data.century & 0b00001111;
	global_digit_num[2] = global_rtc_year >> 4;
	global_digit_num[3] = global_rtc_year & 0b00001111;
	global_digit_num[4] = global_rtc_month >> 4;
	global_digit_num[5] = global_rtc_month & 0b00001111;
	global_digit_num[6] = global_rtc_date >> 4;
	global_digit_num[7] = global_rtc_date & 0b00001111;

	global_digit_L_dot_pwm[0] = 0;
	global_digit_L_dot_pwm[1] = 0;
	global_digit_L_dot_pwm[2] = 0;
	global_digit_L_dot_pwm[3] = 0;
	global_digit_L_dot_pwm[4] = 0;
	global_digit_L_dot_pwm[5] = 0;
	global_digit_L_dot_pwm[6] = 0;
	global_digit_L_dot_pwm[7] = 0;

	// R dot 3, 5 & 7 is controlled by timer2
	global_digit_R_dot_pwm[0] = 0;
	global_digit_R_dot_pwm[1] = 0;	
	global_digit_R_dot_pwm[2] = 0;
	//global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
	global_digit_R_dot_pwm[4] = 0;
	//global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
	global_digit_R_dot_pwm[6] = 0;
	//global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
}

void mode_pressure(void){
	uint8_t temp[DIGIT_NUM+1];

	read_LPS331();
	sprintf((char *)temp, "%8.3lf", global_pressure);

	if(global_config_data.pres_type == MMHG){
		global_digit_num[0] = DIGIT_SUPPRESS;
	}else{
		global_digit_num[0] = temp[0] - 48;
	}
	global_digit_num[1] = temp[1] - 48;
	global_digit_num[2] = temp[2] - 48;
	global_digit_num[3] = temp[3] - 48;
	global_digit_num[4] = DIGIT_SUPPRESS;
	global_digit_num[5] = temp[5] - 48;
	global_digit_num[6] = temp[6] - 48;
	global_digit_num[7] = temp[7] - 48;

	global_digit_L_dot_pwm[0] = 0;
	global_digit_L_dot_pwm[1] = 0;
	global_digit_L_dot_pwm[2] = 0;
	global_digit_L_dot_pwm[3] = 0;
	global_digit_L_dot_pwm[4] = 0;
	global_digit_L_dot_pwm[5] = 0;
	global_digit_L_dot_pwm[6] = 0;
	global_digit_L_dot_pwm[7] = 0;

	global_digit_R_dot_pwm[0] = 0;
	global_digit_R_dot_pwm[1] = 0;
	global_digit_R_dot_pwm[2] = 0;
	global_digit_R_dot_pwm[3] = 0;
	global_digit_R_dot_pwm[4] = DOT_PWM_MAX;
	global_digit_R_dot_pwm[5] = 0;
	global_digit_R_dot_pwm[6] = 0;
	if(global_config_data.pres_type == MMHG){
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
	}else{
		global_digit_R_dot_pwm[7] = 0;
	}
}

void mode_temperature(void){
	uint8_t temp[DIGIT_NUM+1];

	read_LPS331();
	sprintf((char *)temp, "%8.3lf", global_temperature);

	if(global_config_data.temp_type == KEL){
		global_digit_num[0] = temp[1] - 48;
	}else{
		global_digit_num[0] = DIGIT_SUPPRESS;
	}
	global_digit_num[1] = temp[2] - 48;
	global_digit_num[2] = temp[3] - 48;
	global_digit_num[3] = DIGIT_SUPPRESS;
	global_digit_num[4] = temp[5] - 48;
	global_digit_num[5] = temp[6] - 48;
	global_digit_num[6] = temp[7] - 48;
	global_digit_num[7] = DIGIT_SUPPRESS;

/*
	global_digit_L_dot_pwm[0] = 0;
	global_digit_L_dot_pwm[1] = 0;
	global_digit_L_dot_pwm[2] = 0;
	global_digit_L_dot_pwm[3] = 0;
	global_digit_L_dot_pwm[4] = 0;
	global_digit_L_dot_pwm[5] = 0;
	global_digit_L_dot_pwm[6] = 0;
	if(global_config_data.temp_type == FAH){
		global_digit_L_dot_pwm[7] = DOT_PWM_MAX;
	}else{
		global_digit_L_dot_pwm[7] = 0;
	}
*/

	if(global_config_data.temp_type == FAH){
		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
	}else{
		global_digit_R_dot_pwm[0] = 0;
	}
	global_digit_R_dot_pwm[1] = 0;	
	global_digit_R_dot_pwm[2] = 0;
	global_digit_R_dot_pwm[3] = DOT_PWM_MAX;;
	global_digit_R_dot_pwm[4] = 0;
	global_digit_R_dot_pwm[5] = 0;
	global_digit_R_dot_pwm[6] = 0;
	if(global_config_data.temp_type == KEL){
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
	}else{
		global_digit_R_dot_pwm[7] = 0;
	}
}

void mode_shuffle(void){
	if(global_do_shuffle == FALSE){
		global_digit_num[0] = global_temp_shuffle_digit[0];
		global_digit_num[1] = DIGIT_SUPPRESS;
		global_digit_num[2] = global_temp_shuffle_digit[2];
		global_digit_num[3] = global_temp_shuffle_digit[3];
		global_digit_num[4] = global_temp_shuffle_digit[4];
		global_digit_num[5] = global_temp_shuffle_digit[5];
		global_digit_num[6] = global_temp_shuffle_digit[6];
		global_digit_num[7] = global_temp_shuffle_digit[7];

		global_digit_L_dot_pwm[0] = 0;
		global_digit_L_dot_pwm[1] = 0;
		global_digit_L_dot_pwm[2] = 0;
		global_digit_L_dot_pwm[3] = 0;
		global_digit_L_dot_pwm[4] = 0;
		global_digit_L_dot_pwm[5] = 0;
		global_digit_L_dot_pwm[6] = 0;
		global_digit_L_dot_pwm[7] = 0;

		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;	
		global_digit_R_dot_pwm[2] = 0;
		global_digit_R_dot_pwm[3] = 0;
		global_digit_R_dot_pwm[4] = 0;
		global_digit_R_dot_pwm[5] = 0;
		global_digit_R_dot_pwm[6] = 0;
		global_digit_R_dot_pwm[7] = 0;
	}
}

void mode_photo(void){
	global_digit_num[0] = global_config_data.photo_digit[0];
	global_digit_num[1] = global_config_data.photo_digit[1];
	global_digit_num[2] = global_config_data.photo_digit[2];
	global_digit_num[3] = global_config_data.photo_digit[3];
	global_digit_num[4] = global_config_data.photo_digit[4];
	global_digit_num[5] = global_config_data.photo_digit[5];
	global_digit_num[6] = global_config_data.photo_digit[6];
	global_digit_num[7] = global_config_data.photo_digit[7];

	global_digit_L_dot_pwm[0] = global_config_data.photo_L_dot[0];
	global_digit_L_dot_pwm[1] = global_config_data.photo_L_dot[1];
	global_digit_L_dot_pwm[2] = global_config_data.photo_L_dot[2];
	global_digit_L_dot_pwm[3] = global_config_data.photo_L_dot[3];
	global_digit_L_dot_pwm[4] = global_config_data.photo_L_dot[4];
	global_digit_L_dot_pwm[5] = global_config_data.photo_L_dot[5];
	global_digit_L_dot_pwm[6] = global_config_data.photo_L_dot[6];
	global_digit_L_dot_pwm[7] = global_config_data.photo_L_dot[7];

	global_digit_R_dot_pwm[0] = global_config_data.photo_R_dot[0];
	global_digit_R_dot_pwm[1] = global_config_data.photo_R_dot[1];	
	global_digit_R_dot_pwm[2] = global_config_data.photo_R_dot[2];
	global_digit_R_dot_pwm[3] = global_config_data.photo_R_dot[3];
	global_digit_R_dot_pwm[4] = global_config_data.photo_R_dot[4];
	global_digit_R_dot_pwm[5] = global_config_data.photo_R_dot[5];
	global_digit_R_dot_pwm[6] = global_config_data.photo_R_dot[6];
	global_digit_R_dot_pwm[7] = global_config_data.photo_R_dot[7];
}

void mode_test_random(){
		global_digit_num[0] = rand_xor_shift()%10;
		global_digit_num[1] = rand_xor_shift()%10;
		global_digit_num[2] = rand_xor_shift()%10;
		global_digit_num[3] = rand_xor_shift()%10;
		global_digit_num[4] = rand_xor_shift()%10;
		global_digit_num[5] = rand_xor_shift()%10;
		global_digit_num[6] = rand_xor_shift()%10;
		global_digit_num[7] = rand_xor_shift()%10;

		global_digit_R_dot_pwm[0] = rand_xor_shift();
		global_digit_R_dot_pwm[1] = rand_xor_shift();
		global_digit_R_dot_pwm[2] = rand_xor_shift();
		global_digit_R_dot_pwm[3] = rand_xor_shift();
		global_digit_R_dot_pwm[4] = rand_xor_shift();
		global_digit_R_dot_pwm[5] = rand_xor_shift();
		global_digit_R_dot_pwm[6] = rand_xor_shift();
		global_digit_R_dot_pwm[7] = rand_xor_shift();

		global_digit_L_dot_pwm[0] = rand_xor_shift();
		global_digit_L_dot_pwm[1] = rand_xor_shift();
		global_digit_L_dot_pwm[2] = rand_xor_shift();
		global_digit_L_dot_pwm[3] = rand_xor_shift();
		global_digit_L_dot_pwm[4] = rand_xor_shift();
		global_digit_L_dot_pwm[5] = rand_xor_shift();
		global_digit_L_dot_pwm[6] = rand_xor_shift();
		global_digit_L_dot_pwm[7] = rand_xor_shift();
}

void mode_test_pattern(){
	static uint8_t i = 0;

	global_digit_num[0] = i;
	global_digit_num[1] = i;
	global_digit_num[2] = i;
	global_digit_num[3] = i;
	global_digit_num[4] = i;
	global_digit_num[5] = i;
	global_digit_num[6] = i;
	global_digit_num[7] = i;

	if(i & 0b00000001){ // odd
		global_digit_L_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[7] = DOT_PWM_MAX;

		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = 0;
		global_digit_R_dot_pwm[2] = 0;
		global_digit_R_dot_pwm[3] = 0;
		global_digit_R_dot_pwm[4] = 0;
		global_digit_R_dot_pwm[5] = 0;
		global_digit_R_dot_pwm[6] = 0;
		global_digit_R_dot_pwm[7] = 0;
	}else{ // even
		global_digit_L_dot_pwm[0] = 0;
		global_digit_L_dot_pwm[1] = 0;
		global_digit_L_dot_pwm[2] = 0;
		global_digit_L_dot_pwm[3] = 0;
		global_digit_L_dot_pwm[4] = 0;
		global_digit_L_dot_pwm[5] = 0;
		global_digit_L_dot_pwm[6] = 0;
		global_digit_L_dot_pwm[7] = 0;

		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
	}
	i++;
	if(i == 10){i=0;}
}

void mode_test_manual(){
	global_digit_num[0] = global_test_pattern_num;
	global_digit_num[1] = global_test_pattern_num;
	global_digit_num[2] = global_test_pattern_num;
	global_digit_num[3] = global_test_pattern_num;
	global_digit_num[4] = global_test_pattern_num;
	global_digit_num[5] = global_test_pattern_num;
	global_digit_num[6] = global_test_pattern_num;
	global_digit_num[7] = global_test_pattern_num;

	switch(global_test_pattern_dot){
	case BOTH:
		global_digit_L_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[7] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
		break;

	case LEFT:
		global_digit_L_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[7] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = 0;
		global_digit_R_dot_pwm[2] = 0;
		global_digit_R_dot_pwm[3] = 0;
		global_digit_R_dot_pwm[4] = 0;
		global_digit_R_dot_pwm[5] = 0;
		global_digit_R_dot_pwm[6] = 0;
		global_digit_R_dot_pwm[7] = 0;
		break;

	case RIGHT:
		global_digit_L_dot_pwm[0] = 0;
		global_digit_L_dot_pwm[1] = 0;
		global_digit_L_dot_pwm[2] = 0;
		global_digit_L_dot_pwm[3] = 0;
		global_digit_L_dot_pwm[4] = 0;
		global_digit_L_dot_pwm[5] = 0;
		global_digit_L_dot_pwm[6] = 0;
		global_digit_L_dot_pwm[7] = 0;
		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
		break;

	case NONE:
		global_digit_L_dot_pwm[0] = 0;
		global_digit_L_dot_pwm[1] = 0;
		global_digit_L_dot_pwm[2] = 0;
		global_digit_L_dot_pwm[3] = 0;
		global_digit_L_dot_pwm[4] = 0;
		global_digit_L_dot_pwm[5] = 0;
		global_digit_L_dot_pwm[6] = 0;
		global_digit_L_dot_pwm[7] = 0;
		global_digit_R_dot_pwm[0] = 0;
		global_digit_R_dot_pwm[1] = 0;
		global_digit_R_dot_pwm[2] = 0;
		global_digit_R_dot_pwm[3] = 0;
		global_digit_R_dot_pwm[4] = 0;
		global_digit_R_dot_pwm[5] = 0;
		global_digit_R_dot_pwm[6] = 0;
		global_digit_R_dot_pwm[7] = 0;
		break;

	default:
		global_digit_L_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_L_dot_pwm[7] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[0] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[1] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[2] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[3] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[4] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[5] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[6] = DOT_PWM_MAX;
		global_digit_R_dot_pwm[7] = DOT_PWM_MAX;
		break;
	}
}

void mode_test(void){
	switch(global_test_pattern){
	case RANDOM:
		mode_test_random();
		break;
	
	case PATTERN:
		mode_test_pattern();
		break;

	case MANUAL:
		mode_test_manual();
		break;

	default:
		mode_test_random();
		break;
	}
}

void mode_config(void){
	global_digit_num[0] = global_temp_digit[0];
	global_digit_num[1] = global_temp_digit[1];
	global_digit_num[2] = global_temp_digit[2];
	global_digit_num[3] = global_temp_digit[3];
	global_digit_num[4] = global_temp_digit[4];
	global_digit_num[5] = global_temp_digit[5];
	global_digit_num[6] = global_temp_digit[6];
	global_digit_num[7] = global_temp_digit[7];

	global_digit_L_dot_pwm[0] = global_temp_L_dot[0];
	global_digit_L_dot_pwm[1] = global_temp_L_dot[1];
	global_digit_L_dot_pwm[2] = global_temp_L_dot[2];
	global_digit_L_dot_pwm[3] = global_temp_L_dot[3];
	global_digit_L_dot_pwm[4] = global_temp_L_dot[4];
	global_digit_L_dot_pwm[5] = global_temp_L_dot[5];
	global_digit_L_dot_pwm[6] = global_temp_L_dot[6];
	global_digit_L_dot_pwm[7] = global_temp_L_dot[7];

	global_digit_R_dot_pwm[0] = global_temp_R_dot[0];
	global_digit_R_dot_pwm[1] = global_temp_R_dot[1];	
	global_digit_R_dot_pwm[2] = global_temp_R_dot[2];
	global_digit_R_dot_pwm[3] = global_temp_R_dot[3];
	global_digit_R_dot_pwm[4] = global_temp_R_dot[4];
	global_digit_R_dot_pwm[5] = global_temp_R_dot[5];
	global_digit_R_dot_pwm[6] = global_temp_R_dot[6];
	global_digit_R_dot_pwm[7] = global_temp_R_dot[7];
}

void update_mode(uint8_t mode){
	static uint8_t old_mode = MODE_TIME;

	if(old_mode != mode){
		eep_update_mode(mode);
		old_mode = mode;
	}

	if(global_do_shuffle == FALSE){ // not change @ shuffle mode
		switch(mode){
		case MODE_TIME:
			if(global_config_mode == FALSE){
				mode_time();
			}else{
				mode_config();
			}
			break;

		case MODE_DATE:
			if(global_config_mode == FALSE){
				mode_date();
			}else{
				mode_config();
			}
			break;

		case MODE_PRESSURE:
			mode_pressure();
			break;

		case MODE_TEMPERATURE:
			mode_temperature();
			break;

		case MODE_SHUFFLE:
			mode_shuffle();
			break;

		case MODE_PHOTO:
			if(global_config_mode == FALSE){
				mode_photo();
			}else{
				mode_config();
			}
			break;

		case MODE_TEST:
			mode_test();
			break;

		default:
			eep_update_mode(MODE_TIME);
			break;
		}
	}	
}

void increase_mode(void){
	if(global_config_data.mode < MODE_TEST){
		global_config_data.mode++;
	}else{
		global_config_data.mode = 0;
	}
	update_mode(CURRENT_MODE);
}

void decrease_mode(void){
	if(global_config_data.mode > 0){
		global_config_data.mode--;
	}else{
		global_config_data.mode = MODE_TEST;
	}
	update_mode(CURRENT_MODE);
}
